import docker
from rich import print
from rich.progress import Progress
from config import GRAFANA_ENABLED, GRAFANA_SERVER_DOCKER_IMAGE, GRAFANA_SERVER_PORT, GRAFANA_CONTAINER_NAME, GRAFANA_DATA_DIR, GRAFANA_PROVISIONING_DIR
from config import REDIS_ENABLED, REDIS_SERVER_DOCKER_IMAGE, REDIS_SERVER_PORT, REDIS_CONTAINER_NAME, DOCKER_NETWORK
from util.general import check_requirements

tasks = {}

def docker_image_pull_with_progress(client, image_name):

    # Show task progress (red for download, green for extract)
    def show_progress(line, progress):
        if line['status'] == 'Downloading':
            id = f'[red][Download {line["id"]}]'
        elif line['status'] == 'Extracting':
            id = f'[green][Extract  {line["id"]}]'
        else:
            print(line)
            # skip other statuses
            return

        if id not in tasks.keys():
            tasks[id] = progress.add_task(f"{id}", total=line['progressDetail']['total'])
        else:
            progress.update(tasks[id], completed=line['progressDetail']['current'])


    print(f'Pulling image: {image_name}')
    with Progress() as progress:
        resp = client.api.pull(image_name, stream=True, decode=True)
        for line in resp:
            show_progress(line, progress)

def docker_container_run_triton(
    client:docker.DockerClient,     # docker.from_env()
    image,                          # image name
    container_name,                 # container name
    command,                        # command to run in container
    stdout=False,                   # disable logs
    stderr=False,                   # disable stderr logs
    detach=True,                    # detached mode - daemon
    remove=True,                    # remove fs after exit
    auto_remove=True,               # remove fs if container fails
    device_requests=None,           # pass GPU
    network_mode='host',            # --net=host
    volumes=[],                     # -v ./models-repo:/models
    ports={}                        # -p 8000:8000 etc (needed for mac)
):
    ctainer = None
    container_type = 'MODEL_SERVER'
    try:
        print('Try model-server with NVIDIA runtime & GPUs.')
        ctainer = client.containers.run(
            image=image,
            name=container_name,
            command=command,
            stdout=stdout,
            stderr=stderr,
            detach=detach,
            remove=remove,
            auto_remove=auto_remove,
            runtime='nvidia',   # Use nvidia-container-runtime
            device_requests=device_requests,
            # network_mode=network_mode,
            volumes=volumes,
            ports=ports
        )
        container_type = 'NVIDIA Runtime + GPU'
    except Exception as ex:
        ctainer = None
        print('.... Failed.')

    if ctainer is None:
        print('Try model-server without NVIDIA runtime + CPUs')

        try:
            ctainer = client.containers.run(
                image=image,
                name=container_name,
                command=command,
                stdout=stdout,
                stderr=stderr,
                detach=detach,
                remove=remove,
                auto_remove=auto_remove,
                device_requests=device_requests,
                # network_mode=network_mode,
                volumes=volumes,
                ports=ports
            )
            container_type = 'NVIDIA Runtime + CPU'
        except Exception as ex:
            ctainer = None
            print('.... Failed.')

    if ctainer is None:
        print('Try model-server with default runtime + CPU')

        try:
            ctainer = client.containers.run(
                image=image,
                name=container_name,
                command=command,
                stdout=stdout,
                stderr=stderr,
                detach=detach,
                remove=remove,
                auto_remove=auto_remove,
                # network_mode=network_mode,
                volumes=volumes,
                ports=ports
            )
            container_type = 'Default Runtime + CPU'
        except Exception as ex:
            ctainer = None
            print('.... Failed.')

    if ctainer is None:
        print('ERROR: Giving up.')
        return None
    else:
        print('.... Success.')
        print(f'Started model server successfully: {container_type}')
        return ctainer

def docker_container_is_running(container_name):
    client = docker.from_env()
    containers = client.containers.list(all=True)
    for container in containers:
        if container.name == container_name:
            return container.status == 'running'
    return False

def docker_network_is_running(network_name):
    client = docker.from_env()
    networks = client.networks.list()
    for network in networks:
        if network.name == network_name:
            return True
    return False

def docker_network_start(network_name):
    client = docker.from_env()
    if docker_network_is_running(network_name=network_name):
        print(f'{network_name} network already running.')
        return
    print(f'Starting {network_name} network...')
    client.networks.create(network_name, driver='bridge')

def docker_network_join(network_name, container_name):
    client = docker.from_env()
    network = client.networks.get(network_name)
    container = client.containers.get(container_name)
    network.connect(container)

def docker_container_start(
    container_name:str,
    image:str,
    portmap:dict=None,
    envmap:dict=None,
    network_name:str=None,
    volmap:dict=[],
    command:str=None):
    '''
    Start a docker container.

    Args:
        container_name (str): Name of container
        image (str): Name of image
        portmap (dict): Port mapping. Example: {8000:8000, 8001:8001, 8002:8002}
        envmap (dict): Environment variables. Example: {'GF_AUTH_ANONYMOUS_ORG_ROLE': 'Admin'}
        volmap (list): Volume mapping. Example: ['./models-repo:/models']
        network_name (str): Name of network to create + join
        command (str): Command to run in container
    '''
    print('- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -')

    # If container already running, do nothing
    if docker_container_is_running(container_name=container_name):
        print(f'{container_name} container already running.')
        return

    # Pull image
    print(f'Pulling {image} image...')
    client = docker.from_env()
    docker_image_pull_with_progress(client=client, image_name=image)

    # Ports mapping
    if portmap is not None:
        ports = []
        for k in portmap.keys():
            ports.append(f'-p {k}:{portmap[k]}')
        ports = ' '.join(ports)
    else:
        ports = ''

    # Environment variables
    if envmap is not None:
        envs = []
        for k in envmap.keys():
            envs.append(f'-e {k}={envmap[k]}')
        envs = ' '.join(envs)
    else:
        envs = ''

    # Volume mapping
    if volmap is not None:
        volumes = []
        for k in volmap:
            volumes.append(f'-v {k}')
        volumes = ' '.join(volumes)
    else:
        volumes = ''

    # Network name
    if network_name is not None:
        print(f'Joining network...{network_name}')
        docker_network_start(network_name=network_name)

    # Command to run
    if command is None:
        command = ''

    # Run container
    print(f'Running {image} container...')

    cmd_line = f'docker run -d --rm --name {container_name} {ports} {envs} {volumes} {image} {command}'
    cmd_line_hint = False
    try:
        container = client.containers.run(
            image=image,
            name=container_name,
            detach=True,
            remove=True,
            auto_remove=True,
            ports=portmap,
            environment=envmap,
            volumes=volmap,
            network=network_name,
            command=command
        )
    except docker.errors.APIError as e:
        print(f'Error: {e}')
        print(f'{image} container already running. Skipping.')
        cmd_line_hint = False
    except docker.errors.ImageNotFound as e:
        print(f'Error: {e}')
        print(f'{image} image not found. Skipping.')
        cmd_line_hint = True
    except docker.errors.ContainerError as e:
        print(f'Error: {e}')
        print(f'{image} container error. Skipping.')
        cmd_line_hint = True
    except docker.errors.DockerException as e:
        print(f'Error: {e}')
        print(f'{image} docker error. Skipping.')
        cmd_line_hint = True
    except Exception as e:
        print(f'Error: {e}')
        print(f'{image} error. Skipping.')
        cmd_line_hint = True

    # Print command line hint
    if cmd_line_hint:
        print('You can run the following command line to start the {image} container:')
        print(cmd_line)
    else:
        pass

        #     docker_network_join(network_name=network_name, container_name=container_name)

    print('- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -')


def docker_container_stop(container_name):
    print('- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -')

    # Stop container
    print(f'Stopping {container_name} container...')
    cmd_hint = False
    cmd_line = f'docker stop {container_name}'

    client = docker.from_env()
    try:
        container = client.containers.get(container_name)
        container.stop()
    except docker.errors.NotFound as e:
        print(f'{container_name} container not found. Skipping.')
        cmd_hint = True
    except docker.errors.APIError as e:
        print(f'Error: {e}')
        print(f'{container_name} container API error. Skipping.')
        cmd_hint = True
    except docker.errors.DockerException as e:
        print(f'Error: {e}')
        print(f'{container_name} docker error. Skipping.')
        cmd_hint = True
    except Exception as e:
        print(f'Error: {e}')
        print(f'{container_name} error. Skipping.')
        cmd_hint = True

    # Print command line hint
    if cmd_hint:
        print(f'You can run the following command line to stop the {container_name} container:')
        print(cmd_line)

    print('- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -')


def redis_container_start():
    # If redis is not enabled, do nothing
    if REDIS_ENABLED == False:
        print('Redis is not enabled. Skipping installation.')
        return

    # Start redis container
    docker_container_start(
        container_name=REDIS_CONTAINER_NAME,
        image=REDIS_SERVER_DOCKER_IMAGE,
        network_name=DOCKER_NETWORK,
        portmap={6379:REDIS_SERVER_PORT}
        )

def redis_container_stop():
    # If redis is not enabled, do nothing
    if REDIS_ENABLED == False:
        print('Redis is not enabled. Skip redis container stop.')

    # Stop redis container
    docker_container_stop(container_name=REDIS_CONTAINER_NAME)

def redis_python_install():
    # If redis is not enabled, do nothing
    if REDIS_ENABLED == False:
        print('Redis is not enabled. Skipping installation.')
        return

    # Install redis python package
    print('Installing redis python package...')
    cmd_hint = False
    cmd_line = f'pip install redis'

    try:
        check_requirements(['redis'], install=True)
    except Exception as e:
        print(f'Error: {e}')
        print('Redis python package installation failed. Skipping.')
        cmd_hint = True
        return

    # Print command line hint
    if cmd_hint:
        print('You can run the following command line to install the redis python package:')
        print(cmd_line)


def redis_python_package_installed():
    return 'redis' not in check_requirements('redis', install=False)


def grafana_container_start():
    # If grafana is not enabled, do nothing
    if GRAFANA_ENABLED == False:
        print('Grafana is not enabled. Skipping installation.')
        return

    # Grafana default env parameters
    grafana_env = {
        'GF_AUTH_ANONYMOUS_ORG_ROLE': 'Admin',
        'GF_AUTH_ANONYMOUS_ENABLED': True,
        'GF_AUTH_BASIC_ENABLED': False,
        'GF_ENABLE_GZIP': True,
        'GF_USERS_DEFAULT_THEME': 'dark',
        'GF_DEFAULT_APP_MODE': 'development',
        'GF_INSTALL_PLUGINS': 'redis-datasource'
    }

    # Start grafana container
    docker_container_start(
        container_name=GRAFANA_CONTAINER_NAME,
        image=GRAFANA_SERVER_DOCKER_IMAGE,
        network_name=DOCKER_NETWORK,
        portmap={3000:GRAFANA_SERVER_PORT},
        envmap=grafana_env,
        volmap=[f'{GRAFANA_DATA_DIR}:/var/lib/grafana', f'{GRAFANA_PROVISIONING_DIR}:/etc/grafana/provisioning/datasources']
        )

def grafana_container_stop():
    # If grafana is not enabled, do nothing
    if GRAFANA_ENABLED == False:
        print('Grafana is not enabled. Skip grafana container stop.')

    # Stop grafana container
    docker_container_stop(container_name=GRAFANA_CONTAINER_NAME)


if __name__ == '__main__':
    # Pull a large image
    # client = docker.from_env()
    # IMAGE_NAME = 'bitnami/pytorch'
    # docker_image_pull_with_progress(client, IMAGE_NAME)

    # check if container is running
    redis_is_running = docker_container_is_running('redis')
    print(f'redis_is_running: {redis_is_running}')



from rich import print
import time

import sys
from pathlib import Path
FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

from util.general import LOGGER
from scenarios import Scenario
from config import TRITON_HTTP_URL

class SmokingDetection(Scenario):
    def __init__(self, scenario_name, camera_name=0, events=None, triton_url=TRITON_HTTP_URL):

        from models.triton_client_yolov5 import yolov5_triton
        self.model = yolov5_triton(triton_url, scenario_name)
        super().__init__(scenario_name, camera_name, events, triton_url)


    def start(self, camera_name=0):
        '''
        Stream processing

        When running a scenario - the caller can specify any specific camera.
        '''

        import cv2
        stream = camera_name

        print(f'Opening capture for {stream}')
        video = cv2.VideoCapture(stream)

        while True:
            # Do processing
            ret, frame = video.read()
            if ret is False:
                LOGGER.error('ERROR: reading from video frame')
                time.sleep(1)
                continue

            # Detect PPE
            results = self.model(frame, size=640)  # batched inference
            results.print()
            results.show()
            # if result contains people detected with smoking - then fire an event.
            # For now fire-an-event == print the event details.

            # if stop_evt is set, then break
            if self.stop_evt.is_set():
                break




import os
import sys
from pathlib import Path
import unittest

FILE = Path(__file__).resolve()
ROOT = FILE.parents[1] # visionai/visionai
PKGDIR = FILE.parents[2] # visionai folder
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

from config import VISIONAI_EXEC
from util.general import WorkingDirectory, invoke_cmd

class TestInvokeCliCamera(unittest.TestCase):
    @WorkingDirectory(PKGDIR)
    def setUp(self):
        # Initialize visionai package before running tests
        output = invoke_cmd(f'{VISIONAI_EXEC} init')

    @WorkingDirectory(PKGDIR)
    def test_invoke_camera(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} camera')
        assert 'Error' in output
        assert 'Missing command' in output

    @WorkingDirectory(PKGDIR)
    def test_invoke_camera_help(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} camera --help')
        assert 'Usage' in output
        assert 'Commands' in output
        assert 'add' in output
        assert 'list' in output
        assert 'remove' in output

    @WorkingDirectory(PKGDIR)
    def test_invoke_camera_add_help(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} camera add --help')
        assert 'Usage' in output
        assert 'camera' in output
        assert 'add' in output
        assert '--name' in output
        assert '--uri' in output
        assert '--description' in output
        assert '--help' in output

    @WorkingDirectory(PKGDIR)
    def test_invoke_camera_add_remove(self):
        # cleanup prior configuration
        output = invoke_cmd(f'{VISIONAI_EXEC} camera reset --confirm')

        # add camera
        output = invoke_cmd(f'{VISIONAI_EXEC} camera add --name TEST-999 --uri youtube.com --description "Test camera"')
        assert 'Success' in output

        # list camera
        output = invoke_cmd(f'{VISIONAI_EXEC} camera list')
        assert "'name': 'TEST-999'" in output
        assert "'uri': 'youtube.com'" in output

        # remove camera
        output = invoke_cmd(f'{VISIONAI_EXEC} camera remove --name TEST-999')
        assert 'Success' in output

        # list camera
        output = invoke_cmd(f'{VISIONAI_EXEC} camera list')
        assert "'name': 'TEST-999'" not in output

    @WorkingDirectory(PKGDIR)
    def test_invoke_camera_add_scenario_help(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} camera add-scenario --help')
        assert 'Usage' in output
        assert 'scenario' in output
        assert 'add' in output
        assert '--camera' in output
        assert '--scenario' in output
        assert '--help' in output

    @WorkingDirectory(PKGDIR)
    def test_invoke_camera_remove_scenario_help(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} camera remove-scenario --help')
        assert 'Usage' in output
        assert 'scenario' in output
        assert 'remove' in output
        assert '--scenario' in output
        assert '--camera' in output
        assert '--help' in output

    @WorkingDirectory(PKGDIR)
    def test_invoke_camera_list_scenario_help(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} camera list-scenario --help')
        assert 'Usage' in output
        assert 'scenario' in output
        assert '--camera' in output
        assert '--help' in output

    @WorkingDirectory(PKGDIR)
    def test_invoke_camera_list_scenarios_help(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} camera list-scenarios --help')
        assert 'Usage' in output
        assert 'scenario' in output
        assert '--camera' in output
        assert '--help' in output

    @WorkingDirectory(PKGDIR)
    def test_invoke_camera_list_scenarios_all(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} camera list-scenarios')
        assert 'scenarios for all cameras' in output.lower()

    @WorkingDirectory(PKGDIR)
    def test_invoke_camera_list_scenario_all(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} camera list-scenario')
        assert 'scenarios for all cameras' in output.lower()

    @WorkingDirectory(PKGDIR)
    def test_invoke_camera_list_scenario_for_camera(self):
        # reset all cameras
        output = invoke_cmd(f'{VISIONAI_EXEC} camera reset --confirm')

        # add camera
        output = invoke_cmd(f'{VISIONAI_EXEC} camera add --name TEST-999 --uri youtube.com --description "Test camera"')
        assert 'Success' in output

        # add scenarios for camera
        output = invoke_cmd(f'{VISIONAI_EXEC} camera add-scenario --camera TEST-999 --scenario smoke-and-fire-detection')
        assert 'Scenario smoke-and-fire-detection successfully added for camera TEST-999' in output
        output = invoke_cmd(f'{VISIONAI_EXEC} camera add-scenario --camera TEST-999 --scenario ppe-detection')
        assert 'Scenario ppe-detection successfully added for camera TEST-999' in output

        # list cameras for scenario
        output = invoke_cmd(f'{VISIONAI_EXEC} camera list-scenarios --camera TEST-999')
        assert "'name': 'smoke-and-fire-detection'" in output
        assert "'name': 'ppe-detection'" in output
        assert "scenarios for camera: TEST-999" in output

        # reset everything
        output = invoke_cmd(f'{VISIONAI_EXEC} camera reset --confirm')
        assert 'Camera configuration reset' in output

    @WorkingDirectory(PKGDIR)
    def test_invoke_camera_scenario_add_remove_list(self):
        # cleanup (prior test failures)
        output = invoke_cmd(f'{VISIONAI_EXEC} camera reset --confirm')

        # add camera
        output = invoke_cmd(f'{VISIONAI_EXEC} camera add --name TEST-999 --uri youtube.com --description "Test camera"')
        assert 'Successfully added camera: TEST-999' in output

        # add scenario
        output = invoke_cmd(f'{VISIONAI_EXEC} camera add-scenario --camera TEST-999 --scenario smoke-and-fire-detection')
        assert 'Scenario' in output
        assert 'smoke-and-fire-detection' in output
        assert 'added for camera' in output

        # list scenario
        output = invoke_cmd(f'{VISIONAI_EXEC} camera list-scenario --camera TEST-999')
        assert "scenarios for camera" in output
        assert "'cam': 'TEST-999'" in output
        assert "'name': 'smoke-and-fire-detection'" in output

        # remove scenario
        output = invoke_cmd(f'{VISIONAI_EXEC} camera remove-scenario --camera TEST-999 --scenario smoke-and-fire-detection')
        assert 'Successfully removed scenario: smoke-and-fire-detection' in output

        # list scenario
        output = invoke_cmd(f'{VISIONAI_EXEC} camera list-scenario  --camera TEST-999')
        assert "scenarios for camera" in output
        assert 'TEST-999' in output
        assert "'name': 'smoke-and-fire-detection'" not in output

        # remove camera
        output = invoke_cmd(f'{VISIONAI_EXEC} camera remove --name TEST-999')
        assert "Successfully removed camera: TEST-999" in output

        # list scenarios for deleted cam
        output = invoke_cmd(f'{VISIONAI_EXEC} camera list-scenario  --camera TEST-999')
        assert "camera not found: TEST-999" in output

        # list scenario for all
        output = invoke_cmd(f'{VISIONAI_EXEC} camera list-scenarios')
        assert "scenarios for all cameras" in output
        assert "[]" in output


if __name__ == '__main__':
    unittest.main()



import os
import sys
from pathlib import Path
import unittest

FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
PKGDIR = FILE.parents[2] # visionai dir
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

from util.general import WorkingDirectory, invoke_cmd
from config import VISIONAI_EXEC

class TestInvokeCliMain(unittest.TestCase):
    @WorkingDirectory(PKGDIR)
    def setUp(self):
        # Initialize visionai package before running tests
        output = invoke_cmd(f'{VISIONAI_EXEC} init')

    @WorkingDirectory(PKGDIR)
    def test_invoke_main(self):
        output = invoke_cmd(f'{VISIONAI_EXEC}')
        assert 'Error' in output
        assert 'Missing command' in output

    @WorkingDirectory(PKGDIR)
    def test_invoke_main_help(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} --help')
        assert 'VisionAI Toolkit' in output
        assert 'docs.visionify.ai' in output

    @WorkingDirectory(PKGDIR)
    def test_invoke_main_version(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} --version')
        assert 'VisionAI Toolkit Version' in output

    @WorkingDirectory(PKGDIR)
    def test_invoke_main_verbose(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} --verbose')
        assert 'Enabling verbose logging' in output

if __name__ == '__main__':
    unittest.main()



import os
import platform
import sys
from pathlib import Path
import typer
from typing import Optional

FILE = Path(__file__).resolve()
ROOT = FILE.parents[0]  # visionai/visionai directory
PKGDIR = FILE.parents[1] # visionai directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH
ROOT = Path(os.path.relpath(ROOT, Path.cwd()))  # relative

# Initialize configuration (one-time)
from config import init_config
init_config()

# CLI modules
from cli import scenario_app, camera_app, web_app, auth_app, device_app, pipeline_app, models_app
from cli import stop_cmd, init_cmd, status_cmd

app = typer.Typer()
app.add_typer(auth_app, name='auth', help='Authentication commands')

app.add_typer(device_app, name='device', help='Device commands')
app.add_typer(device_app, name='devices', hidden=True)

app.add_typer(scenario_app, name='scenario', help='Add/remove scenarios to camera')
app.add_typer(scenario_app, name='scenarios', hidden=True)

app.add_typer(camera_app, name='camera', help='Add/remove/manage cameras')
app.add_typer(camera_app, name='cameras', hidden=True)

app.add_typer(web_app, name='web', help='Start/stop web server')
app.add_typer(web_app, name='ui', hidden=True)

app.add_typer(pipeline_app, name='pipeline', help='Manage pipelines')
app.add_typer(pipeline_app, name='pipelines', hidden=True)

app.add_typer(models_app, name='model', help='Manage models')
app.add_typer(models_app, name='models', hidden=True)

@app.command('init')
def init():
    '''
    Initialize VisionAI library

    Initialize VisionAI library. This takes care of downloading
    and starting all required container dependencies. This includes:
    - Redis
    - Grafana
    - Triton Model Server
    - VisionAI Web Service Application
    - VisionAI Web API Service

    '''
    init_cmd()

@app.command('stop')
def stop():
    '''
    Stop all running containers.

    Use this function to stop all running containers.
    Example Usage:
    $ visionai stop
    $ visionai stop --help
    '''
    stop_cmd()

@app.command('status')
def status():
    '''
    Print status of all running containers.

    Use this function to print status of all running containers.
    Example Usage:
    $ visionai status
    $ visionai status --help
    '''
    status_cmd()

# Single-source for version
import pkg_resources

try:
    __version__ = pkg_resources.get_distribution('visionai').version
except:
    __version__ = "0.0.0"

__verbose__ = False

def verbose_callback(value: bool):
    ''' Enable verbose logging '''
    if value:
        print('Enabling verbose logging')
        __verbose__ = True

def version_callback(value: bool):
    ''' Print version & exit '''
    if value:
        print(f'VisionAI Toolkit Version: {__version__}')
        raise typer.Exit()

@app.callback()
def app_callback(
    verbose: Optional[bool]=typer.Option(False, '--verbose', callback=verbose_callback, is_eager=True),
    version: Optional[bool]=typer.Option(False, '--version', callback=version_callback, is_eager=True)
):
    '''
    VisionAI Toolkit

    VisionAI tookit provides a large number of ready-to-deploy scenarios
    built using latest computer vision frameworks. Supports many of the
    common workplace health and safety use-cases.

    Start by exploring scenarios through visionai scenario list command.
    After that, you can create a pipeline through the pipeline commands.
    Once a pipeline is configured, you can run the pipeline on the
    any number of cameras.

    Running the toolkit does assume a NVIDIA GPU powered machine for
    efficient performance. Please see the system requirements on the
    documentation.

    You can instead opt to install it through Azure Managed VM, with
    preconfigured machines & recommended hardware support. You can
    find information about this on our documentation website.

    Visit https://docs.visionify.ai for more details.
    '''

if __name__ == "__main__":
    app()


import os
import sys
import re
import platform
from datetime import datetime
import coloredlogs, logging
from typing import Optional
from subprocess import check_output, STDOUT
from logging.handlers import RotatingFileHandler
import inspect
import pkg_resources as pkg
import hashlib
import time
import contextlib
import yaml


from pathlib import Path
FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # edge-inference ROOT folder
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

from util import TryExcept, emojis, WorkingDirectory

def invoke_cmd(cmd):
    try:
        output = check_output(cmd, shell=True, stderr=STDOUT).decode()
    except Exception as ex:
        output = str(ex.output)
    return output

def is_ascii(s=''):
    # Is string composed of all ASCII (no UTF) characters? (note str().isascii() introduced in python 3.7)
    s = str(s)  # convert list, tuple, None, etc. to str
    return len(s.encode().decode('ascii', 'ignore')) == len(s)

class NoColorFormatter(logging.Formatter):
    ANSI_RE = re.compile(r'\x1b\[[0-9;]*m')  # Regex for ansi strings
    MILLI_RE = re.compile(r'(:\d\d),\d\d\d') # Regex for millisec removal
    def format(self, record):
        s = super().format(record)
        s = re.sub(self.ANSI_RE, '', s)
        s = re.sub(self.MILLI_RE, r'\1', s)
        # s = s.encode('ascii', 'ignore').decode()  # Remove emojis
        return s

def create_logger() -> logging.Logger:
    # Use /var/log folder, if not accessible use local
    LOGFILE = '/var/log/vfyapp.log'
    if not os.access(LOGFILE, os.W_OK):
        LOGFILE = ROOT / 'vfyapp.log'
    FORMAT = "%(asctime)s %(filename)20s:%(lineno)3d [%(levelname).3s] %(message)s"
    coloredlogs.install(level='INFO', fmt=FORMAT)

    # Disable azure debug logging
    logging.getLogger('azure').setLevel(logging.WARNING)

    # Create a logger object.
    log = logging.getLogger(__name__)
    log.setLevel(logging.INFO)

    # Output to File (Max log size is 100KB)
    file_hdlr = RotatingFileHandler(LOGFILE, maxBytes=100000, backupCount=3)
    file_hdlr.setLevel(logging.INFO)
    file_hdlr.setFormatter(NoColorFormatter(FORMAT))
    log.addHandler(file_hdlr)
    return log

# Static initialization
LOGGER = create_logger()

def check_python(minimum='3.7.0'):
    # Check current python version vs. required python version
    check_version(platform.python_version(), minimum, name='Python ', hard=True)

def check_version(current='0.0.0', minimum='0.0.0', name='version ', pinned=False, hard=False, verbose=False):
    # Check version vs. required version
    current, minimum = (pkg.parse_version(x) for x in (current, minimum))
    result = (current == minimum) if pinned else (current >= minimum)  # bool
    s = f'WARNING ⚠️ {name}{minimum} is required by YOLOv5, but {name}{current} is currently installed'  # string
    if hard:
        assert result, emojis(s)  # assert min requirements met
    if verbose and not result:
        LOGGER.warning(s)
    return result

@TryExcept()
def check_requirements(requirements=ROOT / 'requirements.txt', exclude=(), install=True, cmds=''):
    # Check installed dependencies meet our inference requirements (pass *.txt file or list of packages or single package str)
    prefix = colorstr('red', 'bold', 'requirements:')
    check_python()  # check python version
    if isinstance(requirements, Path):  # requirements.txt file
        file = requirements.resolve()
        assert file.exists(), f"{prefix} {file} not found, check failed."
        with file.open() as f:
            requirements = [f'{x.name}{x.specifier}' for x in pkg.parse_requirements(f) if x.name not in exclude]
    elif isinstance(requirements, str):
        requirements = [requirements]

    s = ''
    n = 0
    for r in requirements:
        try:
            pkg.require(r)
        except (pkg.VersionConflict, pkg.DistributionNotFound):  # exception if requirements not met
            s += f'"{r}" '
            n += 1

    if s and install:  # check environment variable
        LOGGER.info(f"{prefix} VISIONAI requirement{'s' * (n > 1)} {s}not found, attempting AutoUpdate...")
        try:
            # assert check_online(), "AutoUpdate skipped (offline)"
            LOGGER.info(check_output(f'python3 -m pip install {s} {cmds}', shell=True).decode())
            source = file if 'file' in locals() else requirements
            s = f"{prefix} {n} package{'s' * (n > 1)} updated per {source}\n" \
                f"{prefix} ⚠️ {colorstr('bold', 'Restart runtime or rerun command for updates to take effect')}\n"
            LOGGER.info(s)
        except Exception as e:
            LOGGER.warning(f'{prefix} ❌ {e}')

    return s

def clean_str(s):
    # Cleans a string by replacing special characters with underscore _
    return re.sub(pattern="[|@#!¡·$€%&()=?¿^*;:,¨´><+]", repl="_", string=s)

def colorstr(*input):
    # Colors a string https://en.wikipedia.org/wiki/ANSI_escape_code, i.e.  colorstr('blue', 'hello world')
    *args, string = input if len(input) > 1 else ('blue', 'bold', input[0])  # color arguments, string
    colors = {
        'black': '\033[30m',  # basic colors
        'red': '\033[31m',
        'green': '\033[32m',
        'yellow': '\033[33m',
        'blue': '\033[34m',
        'magenta': '\033[35m',
        'cyan': '\033[36m',
        'white': '\033[37m',
        'bright_black': '\033[90m',  # bright colors
        'bright_red': '\033[91m',
        'bright_green': '\033[92m',
        'bright_yellow': '\033[93m',
        'bright_blue': '\033[94m',
        'bright_magenta': '\033[95m',
        'bright_cyan': '\033[96m',
        'bright_white': '\033[97m',
        'end': '\033[0m',  # misc
        'bold': '\033[1m',
        'underline': '\033[4m'}
    return ''.join(colors[x] for x in args) + f'{string}' + colors['end']

def print_args(args: Optional[dict] = None, show_file=True, show_func=False):
    # Print function arguments (optional args dict)
    x = inspect.currentframe().f_back  # previous frame
    file, _, func, _, _ = inspect.getframeinfo(x)
    if args is None:  # get args automatically
        args, _, _, frm = inspect.getargvalues(x)
        args = {k: v for k, v in frm.items() if k in args}
    try:
        file = Path(file).resolve().relative_to(ROOT)
    except ValueError:
        file = Path(file).stem
    s = (f'{file}: ' if show_file else '') + (f'{func}: ' if show_func else '')
    LOGGER.info(colorstr(s) + ', '.join(f'{k}={v}' for k, v in args.items()))

def file_age(path=__file__):
    # Return days since last file update
    dt = (datetime.now() - datetime.fromtimestamp(Path(path).stat().st_mtime))  # delta
    return dt.days  # + dt.seconds / 86400  # fractional days

def file_date(path=__file__):
    # Return human-readable file modification date, i.e. '2021-3-26'
    t = datetime.fromtimestamp(Path(path).stat().st_mtime)
    return f'{t.year}-{t.month}-{t.day}'

def file_size(path):
    # Return file/dir size (MB)
    mb = 1 << 20  # bytes to MiB (1024 ** 2)
    path = Path(path)
    if path.is_file():
        return path.stat().st_size / mb
    elif path.is_dir():
        return sum(f.stat().st_size for f in path.glob('**/*') if f.is_file()) / mb
    else:
        return 0.0

def file_md5(path):
    path = Path(path)
    if path.is_file():
        hash_md5 = hashlib.md5()
        with open(path, "rb") as f:
            for chunk in iter(lambda: f.read(4096), b""):
                hash_md5.update(chunk)
        return hash_md5.hexdigest()
    else:
        return None

def check_online():
    # Check internet connectivity
    import socket

    def run_once():
        # Check once
        try:
            socket.create_connection(("1.1.1.1", 443), 5)  # check host accessibility
            return True
        except OSError:
            return False

    return run_once() or run_once()  # check twice to increase robustness to intermittent connectivity issues

@TryExcept()
@WorkingDirectory(ROOT)
def git_describe(path=ROOT):
    # Return software version
    try:
        assert (Path(path) / '.git').is_dir()
        return check_output(f'git -C {path} describe', shell=True).decode()[:-1]
    except Exception:
        return ''

def get_system_version():
    return git_describe()

@TryExcept()
@WorkingDirectory(ROOT)
def check_git_status(repo='visionify/edge-inference', branch='main'):
    # edge-inference status check, recommend 'git pull' if code is out of date
    url = f'git@github.com:{repo}.git'
    msg = f', for updates see {url}'
    s = colorstr('github: ')  # string
    assert Path('.git').exists(), s + 'skipping check (not a git repository)' + msg
    assert check_online(), s + 'skipping check (offline)' + msg

    splits = re.split(pattern=r'\s', string=check_output('git remote -v', shell=True).decode())
    matches = [repo in s for s in splits]
    if any(matches):
        remote = splits[matches.index(True) - 1]
    else:
        remote = 'ultralytics'
        check_output(f'git remote add {remote} {url}', shell=True)
    check_output(f'git fetch {remote}', shell=True, timeout=5)  # git fetch
    local_branch = check_output('git rev-parse --abbrev-ref HEAD', shell=True).decode().strip()  # checked out
    n = int(check_output(f'git rev-list {local_branch}..{remote}/{branch} --count', shell=True))  # commits behind
    if n > 0:
        pull = 'git pull' if remote == 'origin' else f'git pull {remote} {branch}'
        s += f"⚠️ YOLOv5 is out of date by {n} commit{'s' * (n > 1)}. Use `{pull}` or `git clone {url}` to update."
    else:
        s += f'up to date with {url} ✅'
    LOGGER.info(s)

@WorkingDirectory(ROOT)
def check_git_info(path=ROOT):
    # YOLOv5 git info check, return {remote, branch, commit}
    check_requirements('gitpython')
    import git
    try:
        repo = git.Repo(path)
        remote = repo.remotes.origin.url  # i.e. 'git@github.com:visionify/edge-inference.git'
        commit = repo.head.commit.hexsha  # i.e. '15b5ee18eb2206d3f9b9b40ea8fe31e43607d8d4'
        try:
            branch = repo.active_branch.name  # i.e. 'main'
        except TypeError:  # not on any branch
            branch = None  # i.e. 'detached HEAD' state
        return {'remote': remote, 'branch': branch, 'commit': commit}
    except git.exc.InvalidGitRepositoryError:  # path is not a git dir
        return {'remote': None, 'branch': None, 'commit': None}

def gpu_mem_stats():
    if platform.system() == 'Darwin':

        # Mac OS - nvidia-smi is not supported.
        check_requirements('psutil', install=True)
        import psutil
        mem = psutil.virtual_memory()

        used_pct_s = f'{mem.used/mem.total * 100:.1f}'
        used_gb_s = f'{mem.used/1024/1024/1024:.0f}'
        total_gb_s = f'{mem.total/1024/1024/1024:.0f}'

        mem_free_pct = mem.available/mem.total * 100
        free_pct_s = f'{mem_free_pct:.1f}'
        free_gb_s = f'{mem.available/1024/1024/1024:.0f}'

        color = 'cyan' if mem_free_pct > 30 else 'red'
        check = '✅' if mem_free_pct > 30 else '❌'

        LOGGER.info(colorstr(color, f'{check} CPU memory [USED]: {used_pct_s}% [{used_gb_s}GB/{total_gb_s}GB]'))
        LOGGER.info(colorstr(color, f'{check} CPU memory [FREE]: {free_pct_s}% [{free_gb_s}GB/{total_gb_s}GB]'))
        return mem_free_pct

    else:
        check_requirements('nvidia-ml-py3', install=True)
        import nvidia_smi

        def sizeof_fmt(num, suffix="B"):
            for unit in ["", "K", "M", "G", "T"]:
                if abs(num) < 1024.0:
                    return f"{num:3.1f}{unit}{suffix}"
                num /= 1024.0
            return f"{num:.1f}Yi{suffix}"

        nvidia_smi.nvmlInit()
        deviceCount = nvidia_smi.nvmlDeviceGetCount()
        for i in range(deviceCount):
            handle = nvidia_smi.nvmlDeviceGetHandleByIndex(i)
            info = nvidia_smi.nvmlDeviceGetMemoryInfo(handle)
            dev = nvidia_smi.nvmlDeviceGetName(handle).decode('utf-8')
            mem_free_pct = 100*info.free/info.total
            color = 'cyan' if mem_free_pct > 30 else 'red'
            check = '✅' if mem_free_pct > 30 else '❌'
            mem_free_pct_s = f'{mem_free_pct:.2f}% free'
            mem_total_s = f'{sizeof_fmt(info.total)}'
            mem_used_s = f'{sizeof_fmt(info.used)}'
            LOGGER.info(colorstr(color, f'{check} GPU {i} [{dev}]: {mem_free_pct_s} [{mem_used_s}/{mem_total_s}]'))

        nvidia_smi.nvmlShutdown()
        return mem_free_pct


class Profile(contextlib.ContextDecorator):
    # YOLOv5 Profile class. Usage: @Profile() decorator or 'with Profile():' context manager
    def __init__(self, t=0.0):
        self.t = t

    def __enter__(self):
        self.start = self.time()
        return self

    def __exit__(self, type, value, traceback):
        self.dt = self.time() - self.start  # delta-time
        self.t += self.dt  # accumulate dt

    def reset(self):
        self.t = 0

    def time(self):
        return time.time()



def copy_attr(a, b, include=(), exclude=()):
    # Copy attributes from b to a, options to only include [...] and to exclude [...]
    for k, v in b.__dict__.items():
        if (len(include) and k not in include) or k.startswith('_') or k in exclude:
            continue
        else:
            setattr(a, k, v)

def increment_path(path, exist_ok=False, sep='', mkdir=False):
    # Increment file or directory path, i.e. runs/exp --> runs/exp{sep}2, runs/exp{sep}3, ... etc.
    path = Path(path)  # os-agnostic
    if path.exists() and not exist_ok:
        path, suffix = (path.with_suffix(''), path.suffix) if path.is_file() else (path, '')

        # Method 1
        for n in range(2, 9999):
            p = f'{path}{sep}{n}{suffix}'  # increment path
            if not os.path.exists(p):  #
                break
        path = Path(p)

        # Method 2 (deprecated)
        # dirs = glob.glob(f"{path}{sep}*")  # similar paths
        # matches = [re.search(rf"{path.stem}{sep}(\d+)", d) for d in dirs]
        # i = [int(m.groups()[0]) for m in matches if m]  # indices
        # n = max(i) + 1 if i else 2  # increment number
        # path = Path(f"{path}{sep}{n}{suffix}")  # increment path

    if mkdir:
        path.mkdir(parents=True, exist_ok=True)  # make directory

    return path

def yaml_load(file='data.yaml'):
    # Single-line safe yaml loading
    with open(file, errors='ignore') as f:
        return yaml.safe_load(f)


def yaml_save(file='data.yaml', data={}):
    # Single-line safe yaml saving
    with open(file, 'w') as f:
        yaml.safe_dump({k: str(v) if isinstance(v, Path) else v for k, v in data.items()}, f, sort_keys=False)



if __name__ == '__main__':
    # Some logging examples.
    LOGGER.debug("this is a debugging message")
    LOGGER.info("this is an informational message")
    LOGGER.warning("this is a warning message")
    LOGGER.error("this is an error message")
    LOGGER.critical("this is a critical message")

    # Testing file utilities.
    LOGGER.info('File age: {}'.format(file_age(__file__)))
    LOGGER.info('File date: {}'.format(file_date(__file__)))
    LOGGER.info('File size: {}'.format(file_size(__file__)))


    # Testing if system is online
    LOGGER.info('System Online: {}'.format(check_online()))

    # Testing Software version
    LOGGER.info('Software version: {}'.format(git_describe()))

    # Testing git info
    LOGGER.info('git info: {}'.format(check_git_info()))

    # Testing git update
    LOGGER.info('git status: {}'.format(check_git_status()))

    # Testing gpu/memory status
    LOGGER.info('GPU Memory: {:.2f}%'.format(gpu_mem_stats()))

    # Testing profiler
    dt = Profile(), Profile(), Profile()
    seen = 0
    for idx in range(20):
        with dt[0]:
            # do processing for dt0
            time.sleep(0.01)

        with dt[1]:
            # do processing for dt1
            time.sleep(0.02)

        with dt[2]:
            # do processing for dt1
            time.sleep(0.03)

        seen += 1

        # Print time (inference-only)
        LOGGER.info(f"{dt[1].dt * 1E3:.1f}ms")

    # Print results
    t = tuple(x.t / seen * 1E3 for x in dt)  # speeds per image
    LOGGER.info(f'Speed: %.1fms pre-process, %.1fms inference, %.1fms postprocess' % t)



import typer
from rich import print
import docker
from rich.console import Console
from rich import print

import sys
from pathlib import Path
FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

from config import *
from util.docker_utils import *

err_console = Console(stderr=True)


if __name__ == '__main__':
    stop_cmd()



import sys
from urllib.parse import urlparse
from pathlib import Path
import importlib
from abc import ABC, abstractmethod
from threading import Thread, Event

FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

from util import TryExcept
from util.general import colorstr, get_system_version, LOGGER, gpu_mem_stats
from config import TRITON_HTTP_URL

class Scenario(ABC):
    def __init__(self, scenario_name, camera_name=0, events=None, triton_url=TRITON_HTTP_URL):
        '''
        Initialize scenario base object.
        '''
        self.scenario_name = scenario_name
        self.camera_name = camera_name
        self.events = events
        self.triton_url = triton_url
        self.stop_evt = Event()

    @abstractmethod
    def start(self, stream=None):
        '''
        Process a camera stream/image/file/web-cam stream

        Implement each scenario as a blocking call. It would be
        called as a thread from the calling process.

        Implement support for handing if stop_evt is set.

        while True:
            # Do processing
            # ...

            # if stop_evt is set, then break
            if self.stop_evt.is_set():
                break

        '''
        pass

    def stop(self):
        '''
        Stop the infernece engine

        This is called when start has been called within a thread etc.
        '''
        self.stop_evt.set()


def load_scenario(scenario_name, camera_name=None, events=None, triton_url=TRITON_HTTP_URL):
    '''
    Load the scenario inference object

    Dynamically identify what is the inference class name based
    on scenario_name and load an object of that class.
    scenario    : smoke-and-fire-detection
    filename    : scenarios/smoke_and_fire_detection.py
    module name : smoke_and_fire_detection
    class name  : SmokeAndFireDetection
    '''

    moduleName = scenario_name.replace('-', '_')
    className  = ''.join(x.capitalize() for x in moduleName.split('_'))
    moduleName = 'scenarios.' + moduleName

    my_module = importlib.import_module(moduleName)
    my_class = getattr(my_module, className)

    # Print remaining GPU memory here.
    LOGGER.info(f'🚀 Loading scenario for {colorstr(scenario_name)}')
    gpu_mem = gpu_mem_stats()
    if gpu_mem < 30:
        LOGGER.warning('⛔ Running low on GPU memory')

    # Create inference object
    # This would be an object with Scenario base class.
    my_object = my_class(scenario_name=scenario_name, camera_name=camera_name, events=events, triton_url=triton_url)
    return my_object




import os
import platform
from pathlib import Path
from urllib.parse import urlparse

import cv2
import numpy as np
import pandas as pd
import requests
import torch
import torch.nn as nn
from PIL import Image
from torch.cuda import amp
import typing

from config import TRITON_HTTP_URL
from util import TryExcept
from util.general import (LOGGER, ROOT, Profile, check_requirements, colorstr, git_describe, file_date,
                            yaml_load, check_version, copy_attr, increment_path)

from util.image_utils import xyxy2xywh, exif_transpose, scale_boxes, letterbox, select_device, non_max_suppression, smart_inference_mode
from models.plots import Annotator, colors, save_one_box


class TritonRemoteModel:
    """ A wrapper over a model served by the Triton Inference Server. It can
    be configured to communicate over GRPC or HTTP. It accepts Torch Tensors
    as input and returns them as outputs.
    """

    def __init__(self, url: str, model_name: str):
        """
        Keyword arguments:
        url: Fully qualified address of the Triton server - for e.g. grpc://localhost:8001 or http://localhost:8000
        """

        parsed_url = urlparse(url)
        if parsed_url.scheme == "grpc":
            from tritonclient.grpc import InferenceServerClient, InferInput

            self.client = InferenceServerClient(parsed_url.netloc)  # Triton GRPC client
            model_serving = False
            avail_models = self.client.get_model_repository_index()
            for model in avail_models:
                if model['name'] == model_name:
                    model_serving = True
                    self.model_name = model['name']
                    self.metadata = self.client.get_model_metadata(self.model_name, as_json=True)
                    break

            if model_serving is False:
                raise Exception(f'Error: model {model_name} not serving')

            def create_input_placeholders() -> typing.List[InferInput]:
                return [
                    InferInput(i['name'], [int(s) for s in i["shape"]], i['datatype']) for i in self.metadata['inputs']]

        else:
            from tritonclient.http import InferenceServerClient, InferInput

            self.client = InferenceServerClient(parsed_url.netloc)  # Triton HTTP client
            avail_models = self.client.get_model_repository_index()
            model_serving = False
            for model in avail_models:
                if model['name'] == model_name:
                    model_serving = True
                    self.model_name = model['name']
                    self.metadata = self.client.get_model_metadata(self.model_name)
                    break

            if model_serving is False:
                raise Exception(f'Error: model {model_name} not serving')

            def create_input_placeholders() -> typing.List[InferInput]:
                return [
                    InferInput(i['name'], [int(s) for s in i["shape"]], i['datatype']) for i in self.metadata['inputs']]

        self._create_input_placeholders_fn = create_input_placeholders

    @property
    def runtime(self):
        """Returns the model runtime"""
        return self.metadata.get("backend", self.metadata.get("platform"))

    def __call__(self, *args, **kwargs) -> typing.Union[torch.Tensor, typing.Tuple[torch.Tensor, ...]]:
        """ Invokes the model. Parameters can be provided via args or kwargs.
        args, if provided, are assumed to match the order of inputs of the model.
        kwargs are matched with the model input names.
        """
        inputs = self._create_inputs(*args, **kwargs)
        response = self.client.infer(model_name=self.model_name, inputs=inputs)
        result = []
        for output in self.metadata['outputs']:
            tensor = torch.as_tensor(response.as_numpy(output['name']))
            result.append(tensor)
        return result[0] if len(result) == 1 else result

    def _create_inputs(self, *args, **kwargs):
        args_len, kwargs_len = len(args), len(kwargs)
        if not args_len and not kwargs_len:
            raise RuntimeError("No inputs provided.")
        if args_len and kwargs_len:
            raise RuntimeError("Cannot specify args and kwargs at the same time")

        placeholders = self._create_input_placeholders_fn()
        if args_len:
            if args_len != len(placeholders):
                raise RuntimeError(f"Expected {len(placeholders)} inputs, got {args_len}.")
            for input, value in zip(placeholders, args):
                input.set_data_from_numpy(value.cpu().numpy())
        else:
            for input in placeholders:
                value = kwargs[input.name]
                input.set_data_from_numpy(value.cpu().numpy())
        return placeholders

class Yolov5Triton(nn.Module):
    # YOLOv5 MultiBackend wrapper class
    def __init__(self, url=TRITON_HTTP_URL, model_name='', fp16=False, labels=[]):
        super().__init__()
        model_name = model_name
        fp16=fp16
        stride = 32  # default stride
        device = select_device()
        cuda = torch.cuda.is_available() and device.type != 'cpu'  # use CUDA

        LOGGER.info(f'Connecting to Triton at {url}/{model_name}')
        check_requirements('tritonclient[all]')
        model = TritonRemoteModel(url=url, model_name=model_name)

        # class names
        names = labels if labels and len(labels) > 0 else {i: f'class{i}' for i in range(999)}

        self.__dict__.update(locals())  # assign all variables to self

    def forward(self, im, augment=False, visualize=False):
        # YOLOv5 MultiBackend inference
        b, ch, h, w = im.shape  # batch, channel, height, width
        if self.fp16 and im.dtype != torch.float16:
            im = im.half()  # to FP16

        y = self.model(im)
        if isinstance(y, (list, tuple)):
            return self.from_numpy(y[0]) if len(y) == 1 else [self.from_numpy(x) for x in y]
        else:
            return self.from_numpy(y)

    def from_numpy(self, x):
        return torch.from_numpy(x).to(self.device) if isinstance(x, np.ndarray) else x

    def warmup(self, imgsz=(1, 3, 640, 640)):
        # Warmup model by running inference once
        im = torch.empty(*imgsz, dtype=torch.half if self.fp16 else torch.float, device=self.device)  # input
        self.forward(im)  # warmup

class AutoShape(nn.Module):
    # YOLOv5 input-robust model wrapper for passing cv2/np/PIL/torch inputs. Includes preprocessing, inference and NMS
    conf = 0.25  # NMS confidence threshold
    iou = 0.45  # NMS IoU threshold
    agnostic = False  # NMS class-agnostic
    multi_label = False  # NMS multiple labels per box
    classes = None  # (optional list) filter by class, i.e. = [0, 15, 16] for COCO persons, cats and dogs
    max_det = 1000  # maximum number of detections per image
    amp = False  # Automatic Mixed Precision (AMP) inference

    def __init__(self, model, verbose=True):
        super().__init__()
        if verbose:
            LOGGER.info('Adding AutoShape... ')
        copy_attr(self, model, include=('yaml', 'nc', 'hyp', 'names', 'stride', 'abc'), exclude=())  # copy attributes
        self.dmb = isinstance(model, Yolov5Triton)  # Is it part of Yolov5 () instance
        self.pt = False
        self.model = model.eval()

    def _apply(self, fn):
        # Apply to(), cpu(), cuda(), half() to model tensors that are not parameters or registered buffers
        self = super()._apply(fn)
        if self.pt:
            m = self.model.model.model[-1] if self.dmb else self.model.model[-1]  # Detect()
            m.stride = fn(m.stride)
            m.grid = list(map(fn, m.grid))
            if isinstance(m.anchor_grid, list):
                m.anchor_grid = list(map(fn, m.anchor_grid))
        return self

    @smart_inference_mode()
    def forward(self, ims, size=640, augment=False, profile=False):
        # Inference from various sources. For size(height=640, width=1280), RGB images example inputs are:
        #   file:        ims = 'data/images/zidane.jpg'  # str or PosixPath
        #   URI:             = 'https://ultralytics.com/images/zidane.jpg'
        #   OpenCV:          = cv2.imread('image.jpg')[:,:,::-1]  # HWC BGR to RGB x(640,1280,3)
        #   PIL:             = Image.open('image.jpg') or ImageGrab.grab()  # HWC x(640,1280,3)
        #   numpy:           = np.zeros((640,1280,3))  # HWC
        #   torch:           = torch.zeros(16,3,320,640)  # BCHW (scaled to size=640, 0-1 values)
        #   multiple:        = [Image.open('image1.jpg'), Image.open('image2.jpg'), ...]  # list of images

        dt = (Profile(), Profile(), Profile())
        with dt[0]:
            if isinstance(size, int):  # expand
                size = (size, size)
            p = next(self.model.parameters()) if self.pt else torch.empty(1, device=self.model.device)  # param
            autocast = self.amp and (p.device.type != 'cpu')  # Automatic Mixed Precision (AMP) inference
            if isinstance(ims, torch.Tensor):  # torch
                with amp.autocast(autocast):
                    return self.model(ims.to(p.device).type_as(p), augment=augment)  # inference

            # Pre-process
            n, ims = (len(ims), list(ims)) if isinstance(ims, (list, tuple)) else (1, [ims])  # number, list of images
            shape0, shape1, files = [], [], []  # image and inference shapes, filenames
            for i, im in enumerate(ims):
                f = f'image{i}'  # filename
                if isinstance(im, (str, Path)):  # filename or uri
                    im, f = Image.open(requests.get(im, stream=True).raw if str(im).startswith('http') else im), im
                    im = np.asarray(exif_transpose(im))
                elif isinstance(im, Image.Image):  # PIL Image
                    im, f = np.asarray(exif_transpose(im)), getattr(im, 'filename', f) or f
                files.append(Path(f).with_suffix('.jpg').name)
                if im.shape[0] < 5:  # image in CHW
                    im = im.transpose((1, 2, 0))  # reverse dataloader .transpose(2, 0, 1)
                im = im[..., :3] if im.ndim == 3 else cv2.cvtColor(im, cv2.COLOR_GRAY2BGR)  # enforce 3ch input
                s = im.shape[:2]  # HWC
                shape0.append(s)  # image shape
                g = max(size) / max(s)  # gain
                shape1.append([int(y * g) for y in s])
                ims[i] = im if im.data.contiguous else np.ascontiguousarray(im)  # update
            shape1 = (640, 640)
            x = [letterbox(im, shape1, auto=False)[0] for im in ims]  # pad
            x = np.ascontiguousarray(np.array(x).transpose((0, 3, 1, 2)))  # stack and BHWC to BCHW
            x = torch.from_numpy(x).to(p.device).type_as(p) / 255  # uint8 to fp16/32

        with amp.autocast(autocast):
            # Inference
            with dt[1]:
                y = self.model(x, augment=augment)  # forward

            # Post-process
            with dt[2]:
                y = non_max_suppression(y if self.dmb else y[0],
                                        self.conf,
                                        self.iou,
                                        self.classes,
                                        self.agnostic,
                                        self.multi_label,
                                        max_det=self.max_det)  # NMS
                for i in range(n):
                    scale_boxes(shape1, y[i][:, :4], shape0[i])

            return Detections(ims, y, files, dt, self.names, x.shape)


class Detections:
    # YOLOv5 detections class for inference results
    def __init__(self, ims, pred, files, times=(0, 0, 0), names=None, shape=None):
        super().__init__()
        d = pred[0].device  # device
        gn = [torch.tensor([*(im.shape[i] for i in [1, 0, 1, 0]), 1, 1], device=d) for im in ims]  # normalizations
        self.ims = ims  # list of images as numpy arrays
        self.pred = pred  # list of tensors pred[0] = (xyxy, conf, cls)
        self.names = names  # class names
        self.files = files  # image filenames
        self.times = times  # profiling times
        self.xyxy = pred  # xyxy pixels
        self.xywh = [xyxy2xywh(x) for x in pred]  # xywh pixels
        self.xyxyn = [x / g for x, g in zip(self.xyxy, gn)]  # xyxy normalized
        self.xywhn = [x / g for x, g in zip(self.xywh, gn)]  # xywh normalized
        self.n = len(self.pred)  # number of images (batch size)
        self.t = tuple(x.t / self.n * 1E3 for x in times)  # timestamps (ms)
        self.s = tuple(shape)  # inference BCHW shape

    def _run(self, pprint=False, show=False, save=False, crop=False, render=False, labels=True, save_dir=Path('')):
        s, crops = '', []
        for i, (im, pred) in enumerate(zip(self.ims, self.pred)):
            s += f'\nimage {i + 1}/{len(self.pred)}: {im.shape[0]}x{im.shape[1]} '  # string
            if pred.shape[0]:
                for c in pred[:, -1].unique():
                    n = (pred[:, -1] == c).sum()  # detections per class
                    s += f"{n} {self.names[int(c)]}{'s' * (n > 1)}, "  # add to string
                s = s.rstrip(', ')
                if show or save or render or crop:
                    annotator = Annotator(im, example=str(self.names))
                    for *box, conf, cls in reversed(pred):  # xyxy, confidence, class
                        label = f'{self.names[int(cls)]} {conf:.2f}'
                        if crop:
                            file = save_dir / 'crops' / self.names[int(cls)] / self.files[i] if save else None
                            crops.append({
                                'box': box,
                                'conf': conf,
                                'cls': cls,
                                'label': label,
                                'im': save_one_box(box, im, file=file, save=save)})
                        else:  # all others
                            annotator.box_label(box, label if labels else '', color=colors(cls))
                    im = annotator.im
            else:
                s += '(no detections)'

            if show:
                cv2.imshow('Output', im)
                key = cv2.waitKey(5)
                if key == 27:
                    import sys
                    print('Exiting.')
                    sys.exit(0)

            if save:
                f = self.files[i]
                im = Image.fromarray(im.astype(np.uint8)) if isinstance(im, np.ndarray) else im  # from np
                im.save(save_dir / f)  # save
                if i == self.n - 1:
                    LOGGER.info(f"Saved {self.n} image{'s' * (self.n > 1)} to {colorstr('bold', save_dir)}")
            if render:
                self.ims[i] = np.asarray(im)
        if pprint:
            s = s.lstrip('\n')
            return f'{s}\nSpeed: %.1fms pre-process, %.1fms inference, %.1fms NMS per image at shape {self.s}' % self.t
        if crop:
            if save:
                LOGGER.info(f'Saved results to {save_dir}\n')
            return crops

    def show(self, labels=True):
        self._run(show=True, labels=labels)  # show results

    def save(self, labels=True, save_dir='runs/detect/exp', exist_ok=False):
        save_dir = increment_path(save_dir, exist_ok, mkdir=True)  # increment save_dir
        self._run(save=True, labels=labels, save_dir=save_dir)  # save results

    def crop(self, save=True, save_dir='runs/detect/exp', exist_ok=False):
        save_dir = increment_path(save_dir, exist_ok, mkdir=True) if save else None
        return self._run(crop=True, save=save, save_dir=save_dir)  # crop results

    def render(self, labels=True):
        self._run(render=True, labels=labels)  # render results
        return self.ims

    def pandas(self):
        # return detections as pandas DataFrames, i.e. print(results.pandas().xyxy[0])
        new = copy(self)  # return copy
        ca = 'xmin', 'ymin', 'xmax', 'ymax', 'confidence', 'class', 'name'  # xyxy columns
        cb = 'xcenter', 'ycenter', 'width', 'height', 'confidence', 'class', 'name'  # xywh columns
        for k, c in zip(['xyxy', 'xyxyn', 'xywh', 'xywhn'], [ca, ca, cb, cb]):
            a = [[x[:5] + [int(x[5]), self.names[int(x[5])]] for x in x.tolist()] for x in getattr(self, k)]  # update
            setattr(new, k, [pd.DataFrame(x, columns=c) for x in a])
        return new


    def tolist(self):
        # return a list of Detections objects, i.e. 'for result in results.tolist():'
        r = range(self.n)  # iterable
        x = [Detections([self.ims[i]], [self.pred[i]], [self.files[i]], self.times, self.names, self.s) for i in r]
        # for d in x:
        #    for k in ['ims', 'pred', 'xyxy', 'xyxyn', 'xywh', 'xywhn']:
        #        setattr(d, k, getattr(d, k)[0])  # pop out of list
        return x

    def print(self):
        LOGGER.info(self.__str__())

    def __len__(self):  # override len(results)
        return self.n

    def __str__(self):  # override print(results)
        return self._run(pprint=True)  # print results

    def __repr__(self):
        return f'YOLOv5 {self.__class__} instance\n' + self.__str__()





import os
import sys
from pathlib import Path
import unittest

FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
PKGDIR = FILE.parents[2] # visionai dir
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

from config import VISIONAI_EXEC
from util.general import WorkingDirectory, invoke_cmd

class TestInvokeCliScenario(unittest.TestCase):
    @WorkingDirectory(PKGDIR)
    def setUp(self):
        # Initialize visionai package before running tests
        output = invoke_cmd(f'{VISIONAI_EXEC} init')

    @WorkingDirectory(PKGDIR)
    def test_invoke_scenario(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} scenario')
        assert 'Error' in output
        assert 'Missing command' in output

    @WorkingDirectory(PKGDIR)
    def test_invoke_scenario_help(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} scenario --help')
        assert 'Usage' in output
        assert 'Commands' in output
        assert 'download' in output
        assert 'list' in output
        assert 'test' in output

    @WorkingDirectory(PKGDIR)
    def test_list_all_scenarios(self):
        # list scenario
        output = invoke_cmd(f'{VISIONAI_EXEC} scenario list')
        assert 'smoke-and-fire-detection' in output
        assert 'ppe-detection' in output
        assert 'docs.visionify.ai' in output
        assert 'Categories' in output
        assert 'Tags' in output
        assert 'Acc:' in output
        assert 'Rec:' in output
        assert 'F1:' in output
        assert 'Size:' in output

    @WorkingDirectory(PKGDIR)
    def test_download_specific_scenario(self):
        # download specific scenario
        output = invoke_cmd(f'{VISIONAI_EXEC} scenario download smoke-and-fire-detection')
        assert 'download_models' in output
        assert 'Model: smoke-and-fire-detection' in output

    @WorkingDirectory(PKGDIR)
    def test_download_all_scenarios(self):
        # download all scenarios
        output = invoke_cmd(f'{VISIONAI_EXEC} scenario download all')
        assert 'Downloading all configured scenarios' in output

    # This requires a prompt from user. Skipping it in unit-tests.
    # @WorkingDirectory(PKGDIR)
    # def test_download_world_scenarios(self):
    #     # download all scenarios
    #     output = invoke_cmd(f'{VISIONAI_EXEC} scenario download world')
    #     assert 'Downloading all available (world) scenarios' in output


if __name__ == '__main__':
    unittest.main()



import os
import sys
import typer
import time
import json
from typing import List, Dict
from uuid import uuid4
from rich import print, prompt
from rich.progress import track
from pathlib import Path
import requests
from rich.console import Console
from rich.columns import Columns
from rich.panel import Panel


FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

MODELS_REPO = ROOT / 'models-repo'

from config import CONFIG_FILE, SCENARIOS_URL, SCENARIOS_OVERRIDE, TRITON_HTTP_URL
from util.download_models import safe_download_to_folder
from models.triton_client import TritonClient
from scenarios import load_scenario

# scenario app
scenario_app = typer.Typer()

# print primitives
def _scenario_pretty(scenario):
    if scenario is None:
        return Panel('[red]NONE[/red]')

    id = scenario.get('id')
    name = scenario.get('name')

    try:
        version = scenario.get('version')
        overview = scenario.get('overview')[:150] + '...'
        model = scenario.get('models')['latest']['name'] + '-v.' + scenario.get('models')['latest']['version']
        accuracy = scenario.get('models')['latest']['accuracy']
        recall = scenario.get('models')['latest']['recall']
        f1 = scenario.get('models')['latest']['f1']
        datasetSize = scenario.get('models')['latest']['datasetSize']
        docs = scenario.get('docs')
        events = '[b]Events: [/b]' + ' | '.join(scenario.get('events'))
        categories = '[b]Categories: [/b]' + ' | '.join(scenario.get('categories'))
        tags = '[b]Tags: [/b]' + ' | '.join(scenario.get('tags'))
        metrics_txt = f'[magenta]Acc: {accuracy}% | Rec: {recall}% | F1: {f1}%[/magenta]\n' + \
            f'[magenta]Size: {datasetSize} images[/magenta]'
        scenario_txt = f'[blue]{model}[/blue]\n' + \
            f'[grey]{overview}[/grey]\n' + \
            metrics_txt + '\n' +\
            f'[grey]{docs}[/grey]\n' +\
            f'[cyan]{events}[/cyan]\n' + \
            f'[cyan]{categories}[/cyan]\n' +\
            f'[cyan]{tags}[/cyan]'

        return Panel(scenario_txt, title=f'[yellow]{name}[/yellow]', width=60, expand=False)
    except Exception as ex:
        return Panel(f'[red]ERR {name}[/red]')

def get_scenarios() -> List[Dict]:
    '''
    Get all scenarios available in the system
    '''
    if Path(SCENARIOS_OVERRIDE).exists():
        with open(SCENARIOS_OVERRIDE, 'r') as f:
            scenarios = json.load(f)['scenarios']
    else:
        res = requests.get(SCENARIOS_URL)
        scenarios = res.json()['scenarios']
    return scenarios

def print_scenarios(scenarios):
    console = Console()
    scen_pretty = [_scenario_pretty(scen) for scen in scenarios]
    console.print(Panel(Columns(scen_pretty), title='Scenarios'))


@scenario_app.command('list')
def scenario_list():
    '''
    List all scenarios available

    List all scenarios available in the system. This includes scenarios
    that may or maynot be applied to any specific camera.
    '''

    # Get scenarios from server. If override file exists, use that.
    all_scenarios = get_scenarios()
    print_scenarios(all_scenarios)

@scenario_app.command('download')
def scenario_download(
    name: str
    ):
    '''
    Download models for scenarios

    Ex: visionai scenario download ppe-detection  # download ppe-detection scenario
    Ex: visionai scenario download all            # download all configured scenarios for the org
    Ex: visionai scenario download world          # download all available scenarios

    Download models for a given scenario, or download models for
    all scenarios that have been configured.
    '''

    # import TritonClient module
    from models.triton_client import TritonClient

    # Get scenarios from server. If override file exists, use that.
    all_scenarios = get_scenarios()

    models_changed = False
    if name.lower() == 'world':
        print(f'Downloading all available (world) scenarios')
        yN = prompt.Confirm.ask('Are you sure you want to download all scenarios? This may take a lot of space!')
        if yN is True:
            num_scenarios = len(all_scenarios)
            for idx, scen in enumerate(all_scenarios):
                print(f'Downloading {idx+1}/{num_scenarios}...')
                scen_url = scen['models']['latest']['model_url']
                models_changed = safe_download_to_folder(scen_url, MODELS_REPO, overwrite=False)

        else:
            raise typer.Exit()

    elif name.lower() == 'all':
        print(f'Downloading all configured scenarios')
        model_names = set()

        if not os.path.exists(CONFIG_FILE):
            print('No scenarios configured. Please use visionai scenario add first')
            return

        with open(CONFIG_FILE) as f:
            config_data = json.load(f)

        for cam in config_data['cameras']:
            for scen in cam['scenarios']:
                model_names.add(scen['name'])

            for preproc in cam['preprocess']:
                model_names.add(preproc['name'])

        if len(model_names) == 0:
            # No models to download
            print(f'Error: No scenarios configured. Use `visionai scenario add`')

        else:
            # Download requred models
            for scen in all_scenarios:
                scen_name = scen['name']
                scen_url = scen['models']['latest']['model_url']
                if scen_url is None:
                    continue

                if scen_name in model_names:
                    print(f'Model: {scen_name}: {scen_url}')
                    models_changed = safe_download_to_folder(scen_url, MODELS_REPO, overwrite=False)

    else:
        print(f'Downloading models for scenario: {name}')
        download_success = False

        # If a single scenario name is specified.
        for scen in all_scenarios:
            scen_name = scen['name']
            scen_url = scen['models']['latest']['model_url']
            if scen_url is None:
                continue

            if scen_name == name:
                print(f'Model: {scen_name}: {scen_url}')
                models_changed = safe_download_to_folder(scen_url, MODELS_REPO, overwrite=False)
                download_success = True
                break

        if download_success is False:
            print(f'ERROR: Unable to find scenario: {name}')

    # If any new models were downloaded, restart the model server
    if models_changed is True:
        print('Restarting model server...')
        tc = TritonClient()
        tc.stop_model_server()
        tc.start_model_server()
        print('Restarted model server.')

    # Done downloading models.
    print('Done.')


@scenario_app.command('test')
def scenario_test(
    name:str,
    camera: str = typer.Option('0', help='Camera name (default is webcam)')
    ):
    '''
    Run the scenario locally to test it out.

    - Download the model if not available.
    - Pull the model server container image.
    - Start the model server container with this model.
    - Run inference with this model
    '''

    # import TritonClient module
    from models.triton_client import TritonClient

    # Get scenarios from server. If override file exists, use that.
    all_scenarios = get_scenarios()

    scenario_to_test = None
    for scen in all_scenarios:
        if scen['name'] == name:
            scenario_to_test = scen
            break

    if scenario_to_test is None:
        print(f'ERROR: Scenario {name} not found.')
        raise typer.Exit()

    # Check if model server is running
    tc = TritonClient()
    if not tc.is_triton_running():
        print(f'ERROR: Model server is not running. Start with')
        print('[magenta] visionai models serve [/magenta]')
        raise typer.Exit()
    else:
        print('Model-server running.')


    # Check if the specified scenario model is running.
    model_to_test_available = False
    model_name = scenario_to_test['models']['latest']['name']

    model_list = tc.get_models()
    for model_item in model_list:
        if model_item['name'] == model_name:
            model_to_test_available = True
            break

    # Print models being run
    tc.print_models_served()

    if not model_to_test_available:
        print(f'ERROR: Model {name} is not being served by the model-server.')
        print(f'Download models and restart model-server')
        print(f'[magenta]visionai models stop[/magenta]')
        print(f'[magenta]visionai scenario download {name}[/magenta]')
        print(f'[magenta]visionai models serve[/magenta]')
        print(f'[magenta]visionai scenario test {name} [/magenta]')
        raise typer.Exit()
    else:
        print(f'Model {name} is being served.')

    print(f'Loading inference engine for {name}')
    inference_engine = load_scenario(scenario_name=name, camera_name=camera)

    # for test command - block the main thread.
    print(f'Running on default web-cam')
    inference_engine.start()

    #TODO: Provide support for running on any camera


@scenario_app.callback()
def callback():
    '''
    Manage scenarios

    An organization can have multiple scenarios that are installed
    at different places. They may be from different vendors and/or
    maybe using different security surveillance software. Most
    scenarios however do support RTSP, RTMP or HLS streams as an
    output. Please refer to your scenario vendor documentation to
    find this out. This module will help you onboard those scenarios
    on visionai systems by using a simple named instance for each
    scenario.
    '''

if __name__ == '__main__':
    # scenario_app()
    # scenario_list()

    # scenario_remove('smoke-and-fire-detection', 'TEST-999')
    # scenario_add('smoke-and-fire', 'TEST-999')
    # scenario_download(world=True)
    # scenario_download('face-blur')

    # scenario_download('all')

    # scenario_test(name='smoke-and-fire-detection', camera=0)

    # scenario_test(name='face-blur', camera=0)

    scenario_test(name='ppe-detection', camera='0')


from rich import print
import time
import cv2
from PIL import Image
import math
from math import atan
import sys
from pathlib import Path
FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH
from models.plots import Annotator
from util.general import LOGGER
from scenarios import Scenario
from config import TRITON_HTTP_URL
from events.events_engine import EventsEngine
from enum import Enum

class Event(str, Enum):
    DEBUG = 'DEBUG'
    INFO = 'INFO'
    WARNING = 'WARNING'
    ERROR = 'ERROR'
    CRITICAL = 'CRITICAL'

class SlipAndFallDetection(Scenario):
    model = None
    conf_thres=0.25  # confidence threshold
    iou_thres=0.45  # NMS IOU threshold
    classes=None  # filter by class: --class 0, or --class 0 2 3
    agnostic_nms=False  # class-agnostic NMS
    max_det=1000  # maximum detections per image
    line_thickness=3  # bounding box thickness (pixels)
    color = (0, 200, 0)
    thickness = 1 
    device='cpu'  # cuda device, i.e. 0 or 0,1,2,3 or cpu

    def __init__(self, scenario_name, camera_name=0, events=None, triton_url=TRITON_HTTP_URL):
        
        from util.general import check_requirements
        check_requirements('mediapipe', install=True)
        import mediapipe as mp
        self.mp_drawing = mp.solutions.drawing_utils
        self.mp_drawing_styles = mp.solutions.drawing_styles
        self.mp_pose = mp.solutions.pose
        self.f_event =  EventsEngine(use_redis=True)
        from models.triton_client_yolov5 import yolov5_triton
        self.model = yolov5_triton('http://localhost:8000', scenario_name)
        super().__init__(scenario_name, camera_name, events, triton_url)


    def point_check(self, dim, p) : # bl, tr bottom_left, top_right, point
        bl = [dim[0],dim[3]]
        tr = [dim[2],dim[1]]
        # print(bl, tr, p)
        if (p[0] > bl[0] and p[0] < tr[0] and p[1] < bl[1] and p[1] > tr[1]) :
            return True
        else :
            return False

    def findAngle(self, M1, M2):
        try:
            PI = 3.14159265
            angle = abs((M2 - M1) / (1 + M1 * M2))
            ret = atan(angle)
            val = (ret * 180) / PI
            return round(val, 4)
        except:
            return 0.0

    def draw_box(self, image, land_marks):
        cv2.line(image, land_marks[11], land_marks[12], self.color, self.thickness)
        cv2.line(image, land_marks[11], land_marks[23], self.color, self.thickness)
        cv2.line(image, land_marks[12], land_marks[24], self.color, self.thickness)
        cv2.line(image, land_marks[23], land_marks[24], self.color, self.thickness)
        cv2.line(image, land_marks[23], land_marks[25], self.color, self.thickness)
        cv2.line(image, land_marks[24], land_marks[26], self.color, self.thickness)
        cv2.line(image, land_marks[25], land_marks[27], self.color, self.thickness)
        cv2.line(image, land_marks[26], land_marks[28], self.color, self.thickness)
        return image

    def slope(self, x, y):
        if(y[0] - x[0] != 0):
            return (float)(y[1]-x[1])/(y[0]-x[0])
        return 0.0

    def find_angle(self, land_marks):
        ind = [[11,27,2,23],[5,12,24,28]]
        angles = []
        slopee = []
        sl = self.slope(land_marks[11],land_marks[27])
        ag = self.findAngle(sl, 0)
        return ag

    def find_height(self, land_marks):
        top_point = [0,5,8,10]
        bottom_point = [26,28,30,32]
        height = 0
        for h in range(0,3):
            try:
                height = land_marks[bottom_point[h]][1] - land_marks[top_point[h]][1]
                # print("preson height in loop : ",height)
                break
            except:
                continue
        return height

    def frame_edit(self, im0):
        vision_image = cv2.imread("visionify.jpg") # image size = 107x29
        im0[25:54,25:132] = vision_image
        # im0[0:65, 880:1279] = cv2.medianBlur(im0[0:65, 880:1279],35)
        return im0

    def start(self, camera_name=0):
        '''
        Stream processing

        When running a scenario - the caller can specify any specific camera.
        '''

        import cv2
        stream = camera_name

        print(f'Opening capture for {stream}')
        video = cv2.VideoCapture(stream)

        while True:
            # Do processing
            ret, image = video.read()
            if ret is False:
                LOGGER.error('ERROR: reading from video frame')
                time.sleep(1)
                continue
            
            start_time = time.time()
            centroid = []
            top_center = []
            # Load model
            device = 'cpu'
            results = self.model(image, size=640) #, visualize=False)
            stride, names= self.model.stride, self.model.names
            im0 = image
            seen = 0
            det = results.xyxy[0]
            # seen += 1
            annotator = Annotator(im0, line_width=self.line_thickness) #, example=str(names))
            person_count = 0
            if len(det):
                count_i = 0
                param = {"person_id": [], "pose_points":[], "dim":[], "mobile_centeroid": []}
                for *xyxy, conf, cls in reversed(det):
                    c = int(cls)  # integer class
                    label = f'{names[c]} {conf:.2f}'
                    # print("Detected labels : ", label)
                    if 'class0' in label:
                        person_status = "Normal"
                        crop = im0[int(xyxy[1]):int(xyxy[3]),int(xyxy[0]):int(xyxy[2])]
                        param["person_id"].append(person_count)
                        with self.mp_pose.Pose(
                            min_detection_confidence=0.5,
                            min_tracking_confidence=0.5) as pose:
                            crop = cv2.cvtColor(crop, cv2.COLOR_BGR2RGB)
                            results = pose.process(crop)
                            crop= cv2.cvtColor(crop, cv2.COLOR_RGB2BGR)
                            land_marks = []
                            if results.pose_landmarks:
                                for idx, pose_marks in enumerate(results.pose_landmarks.landmark):
                                    h,w = int(xyxy[3]-xyxy[1]), int(xyxy[2]-xyxy[0])
                                    cx,cy = int(pose_marks.x * w) , int(pose_marks.y * h)
                                    land_marks.append([cx,cy])
                                if land_marks != []:
                                    crop = self.draw_box(crop, land_marks)
                                    slopeangle = self.find_angle(land_marks)
                                    if land_marks[11][1] < land_marks[27][1] + 50:
                                        if slopeangle > 35:
                                            cv2.putText(crop, "No Fall", (0,50) , cv2.FONT_HERSHEY_SIMPLEX, 1, (255,0,0), 2, cv2.LINE_AA)
                                            self.f_event.fire_event(Event.INFO, 'OFFICE-01', 'slip-and-fall-detection', 'NO_FALL', {})
                                        else:
                                            cv2.putText(crop, "Fall", (0,50) , cv2.FONT_HERSHEY_SIMPLEX, 1, (255,0,0), 2, cv2.LINE_AA) 
                                            self.f_event.fire_event(Event.WARNING, 'OFFICE-01', 'slip-and-fall-detection', 'FALL', {})
                                    else:
                                        cv2.putText(crop, "Fall", (0,50) , cv2.FONT_HERSHEY_SIMPLEX, 1, (255,0,0), 2, cv2.LINE_AA) 
                                        self.f_event.fire_event(Event.WARNING, 'OFFICE-01', 'slip-and-fall-detection', 'FALL', {})
                                    im0[int(xyxy[1]):int(xyxy[3]),int(xyxy[0]):int(xyxy[2])] = crop
                            param["pose_points"].append(land_marks)
                            param["dim"].append([int(xyxy[0]),int(xyxy[1]),int(xyxy[2]),int(xyxy[3])])
                            annotator.box_label(xyxy, label)
                        person_count+=1
                im0 = annotator.result()
            if self.stop_evt.is_set():
                    break
            cv2.imshow('detection', im0)
            if cv2.waitKey(5) & 0xFF == 27:
                break
        video.release()
        cv2.destroyAllWindows()

def test_slip_and_fall_detection():
    snf = SlipAndFallDetection(scenario_name = 'phone-detection')
    # snf.load_model()
    img = cv2.imread("visionai/models/data/images/fall_12.png")
    # img = Image.open('visionai/models/data/images/no_fall.jpg')
    # img = Path('visionai/models/data/images/no_fall.jpg')
    img  = snf.process_image(img)
    cv2.imwrite('sfd_res_2.jpg', img)


def camera_stream():
    snf = SlipAndFallDetection(scenario_name = 'phone-detection')
    snf.start(camera_name=0)

if __name__ == '__main__':
    # people_taking_picture_detection()
    camera_stream()



from util.general import check_requirements

def opencv_stream():
    check_requirements(['opencv-python', 'imutils'])

    import cv2
    import time
    from imutils.video import WebcamVideoStream

    rtsp_url = "rtsp://username:password@ip_address:port_number/stream_name"
    fourcc = cv2.VideoWriter_fourcc(*"H264")

    src = 0 # 0 for the default camera
    vs = WebcamVideoStream(src).start()
    time.sleep(2.0) # Wait for the camera to warm up

    out = cv2.VideoWriter(rtsp_url, fourcc, 10, (640, 480), True)

    while True:
        frame = vs.read()
        out.write(frame)
        cv2.imshow("Frame", frame)
        key = cv2.waitKey(1) & 0xFF
        if key == ord("q"):
            break

    out.release()
    vs.stop()
    cv2.destroyAllWindows()



import os
import sys
from pathlib import Path
import json
import requests
from rich import print

FILE = Path(__file__).resolve()
ROOT = FILE.parents[0]  # visionai/visionai directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

# Config file
CONFIG_FOLDER = ROOT / 'config'
CONFIG_FILE = ROOT / 'config' / 'config.json'
SCENARIOS_URL = "https://raw.githubusercontent.com/visionify/visionai/main/visionai/scenarios/scenarios.json"
SCENARIOS_OVERRIDE = ROOT / 'config' / 'scenarios-override.json'

# Triton server endpoints
TRITON_HTTP_URL = 'http://localhost:8000'
TRITON_GRPC_URL = 'grpc://localhost:8001'
TRITON_METRICS_URL = 'http://localhost:8002/metrics'

TRITON_SERVER_CONTAINER_NAME = 'visionai-triton'
TRITON_SERVER_DOCKER_IMAGE = 'nvcr.io/nvidia/tritonserver:22.12-py3'
TRITON_SERVER_EXEC = 'tritonserver'
TRITON_SERVER_COMMAND = 'tritonserver --model-repository=/models'
TRITON_MODELS_REPO = ROOT / 'models-repo'

# Redis server configuration
REDIS_ENABLED = True
REDIS_SERVER_DOCKER_IMAGE = 'redis'
REDIS_SERVER_PORT = 6379
REDIS_CONTAINER_NAME = 'visionai-redis'

# Grafana server configuration
GRAFANA_ENABLED = True
GRAFANA_SERVER_DOCKER_IMAGE = 'grafana/grafana'
GRAFANA_SERVER_PORT = 3003
GRAFANA_CONTAINER_NAME = 'visionai-grafana'
GRAFANA_DATA_DIR = ROOT / 'config' / 'grafana-data'
GRAFANA_PROVISIONING_DIR = ROOT / 'config' / 'grafana-provisioning'

# Web application (front-end)
WEB_APP_DOCKER_IMAGE = 'visionify/visionaiweb'
WEB_APP_PORT = 3001
WEB_APP_CONTAINER_NAME = 'visionai-web'

# Web API (back-end)
WEB_API_DOCKER_IMAGE = 'visionify/visionai-api'
WEB_API_PORT = 3002
WEB_API_MODELS_REPO = ROOT / 'models-repo'
WEB_API_CONFIG_FOLDER = ROOT / 'config'
WEB_API_CONTAINER_NAME = 'visionai-api'

# Docker network
DOCKER_NETWORK = 'visionai-network'

# Test stuff
if os.environ.get('VISIONAI_EXEC') == 'visionai':
    VISIONAI_EXEC = 'visionai'
else:
    VISIONAI_EXEC = 'python -m visionai'


def init_config():
    '''
    Set up initial configuration (one-time only)
    '''

    if not os.path.isdir(CONFIG_FOLDER):
        os.makedirs(CONFIG_FOLDER, exist_ok=True)
        print(f'init(): Created config folder: {CONFIG_FOLDER}')

    if not os.path.exists(CONFIG_FILE):
        config_data = {
            'version': '0.1',
            'cameras': []
            }
        with open(CONFIG_FILE, 'w') as f:
            json.dump(config_data, f, indent=4)
        print(f'init(): Created camera configuration: {CONFIG_FILE}')

    if not os.path.isdir(TRITON_MODELS_REPO):
        os.makedirs(TRITON_MODELS_REPO, exist_ok=True)
        print(f'init(): Created models repo: {TRITON_MODELS_REPO}')


    if GRAFANA_ENABLED:
        if not os.path.isdir(GRAFANA_DATA_DIR):
            os.makedirs(GRAFANA_DATA_DIR, exist_ok=True)
            print(f'init(): Created Grafana data directory: {GRAFANA_DATA_DIR}')

        if not os.path.isdir(GRAFANA_PROVISIONING_DIR):
            os.makedirs(GRAFANA_PROVISIONING_DIR, exist_ok=True)
            print(f'init(): Created Grafana provisioning directory: {GRAFANA_PROVISIONING_DIR}')



from .camera_app import camera_app
from .scenario_app import scenario_app
from .auth_app import auth_app
from .device_app import device_app
from .pipeline_app import pipeline_app
from .web_app import web_app
from .models_app import models_app
from .commands import init_cmd, stop_cmd, status_cmd



import os
import sys
import typer
import time
import json
from uuid import uuid4
from rich import print, prompt
from rich.progress import track
from pathlib import Path
import requests

FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

MODELS_REPO = ROOT / 'models-repo'

from config import CONFIG_FILE, SCENARIOS_URL, TRITON_SERVER_DOCKER_IMAGE
from util.general import invoke_cmd
from util.download_models import safe_download_to_folder
from models.triton_client import TritonClient

# Model serve app
models_app = typer.Typer()

@models_app.command('check')
def models_check():
    '''
    Check model-server status & print helpful debug info.

    TODO:
    Goal of the check command is to identify any configuration/dependency
    issues that we can inform to user that he can fix on his end. This could
    be like missing dependency, missing software package, missing driver details
    etc.

    - Check if model-server is running or not.
    - Check if triton-client can access model-server
    - Check what are the models served
    - Print all of this in a pretty manner [checkbox based]
    - Check container logs & show them here.
    - grep container logs for common errors & highlight that in output
    '''
    print('- - - - - - - - - - - - - - - - - - - - - - - - - - -')
    print('Checking model server...')

    # TODO: may need other information for debug
    #
    tc = TritonClient()
    tc.print_models_served()

@models_app.command('start')
@models_app.command('serve')
def models_start():
    '''
    Start serving all available models.

    All models present in the models-repo/ will be served. We use
    triton inference server to serve them. The triton server will be
    at http://localhost:8000, grpc://localhost:8001.

    Please make sure these two ports are not used by anyone else.
    '''
    print('- - - - - - - - - - - - - - - - - - - - - - - - - - -')
    print('Start models...')
    tc = TritonClient()
    tc.start_model_server()
    tc.print_models_served()

@models_app.command('stop')
def models_stop():
    '''
    Stop serving all models.

    This method will stop serving all models. Any inference running
    will all be stopped as well.
    '''
    print('- - - - - - - - - - - - - - - - - - - - - - - - - - -')
    print('Stop models...')
    tc = TritonClient()
    tc.stop_model_server()

@models_app.command('list')
@models_app.command('status')
def models_status():
    '''
    Show the status of serving models

    Shows how many models are being served, metrics for the models etc.
    '''
    print('- - - - - - - - - - - - - - - - - - - - - - - - - - -')
    print('Models status...')
    tc = TritonClient()
    tc.print_models_served()

@models_app.callback()
def callback():
    '''
    Serve models

    Before we can run any scenarios - the models necessary for them
    must be ready. We use Triton inference server to make the
    best use of GPU/CPU resources available on the machine in order
    to serve our models. Any models that are available in
    models-repo folder would be served after this (TODO - only
    serve models configured in scenarios).
    '''

if __name__ == '__main__':
    # models_app()

    models_status()




from rich import print
import time

import sys
from pathlib import Path
FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH
from models.plots import Annotator
from util.general import LOGGER
from scenarios import Scenario
from config import TRITON_HTTP_URL
from events.events_engine import EventsEngine
from enum import Enum

class Event(str, Enum):
    DEBUG = 'DEBUG'
    INFO = 'INFO'
    WARNING = 'WARNING'
    ERROR = 'ERROR'
    CRITICAL = 'CRITICAL'

class PpeDetection(Scenario):
    def __init__(self, scenario_name, camera_name=0, events=None, triton_url=TRITON_HTTP_URL):
        self.f_event =  EventsEngine(use_redis=True)
        from models.triton_client_yolov5 import yolov5_triton
        labels_file = ROOT / 'models-repo' / 'ppe-detection' / 'labels.txt'
        with open(labels_file, 'r') as f:
            labels = [line.strip() for line in f.readlines()]
        self.model = yolov5_triton(triton_url, scenario_name, labels=labels)
        self.model.conf = 0.35
        super().__init__(scenario_name, camera_name, events, triton_url)


    def start(self, camera_name=0):
        '''
        Stream processing

        When running a scenario - the caller can specify any specific camera.
        '''

        import cv2
        stream = camera_name

        print(f'Opening capture for {stream}')
        video = cv2.VideoCapture(stream)

        while True:
            # Do processing
            ret, image = video.read()
            if ret is False:
                LOGGER.error('ERROR: reading from video frame')
                time.sleep(1)
                continue
            
            # Load model
            pay_load = {'Cardboard' : 'No',
                        'Vest' : 'No',
                        'Helmet' : 'No',
                        'Shoes' : 'No',
                        'Gloves' : 'No'
                        }
            results = self.model(image, size=640) #, visualize=False)
            stride, names= self.model.stride, self.model.names
            im0 = image
            
            det = results.xyxy[0]
            # seen += 1
            annotator = Annotator(im0, line_width=3) #, example=str(names))
            
            if len(det):
                for *xyxy, conf, cls in reversed(det):
                    c = int(cls)  # integer class
                    label = f'{names[c]} {conf:.2f}'
                    if 'Cardboard' in label:
                        pay_load['Cardboard'] = 'Yes'
                        annotator.box_label(xyxy, 'Cardboard')
                    elif 'Vest' in label:
                        pay_load['Vest'] = 'Yes'
                        annotator.box_label(xyxy, 'Vest')
                    elif 'Helmet' in label:
                        pay_load['Helmet'] = 'Yes'
                        annotator.box_label(xyxy, 'Helmet')
                    elif 'No Vest' in label:
                        pay_load['Vest'] = 'No Vest'
                        annotator.box_label(xyxy, 'No Vest')
                    elif 'No Helmet' in label:
                        pay_load['Helmet'] = 'No Helmet'
                        annotator.box_label(xyxy, 'No Helmet')
                    elif 'Shoes' in label:
                        pay_load['Shoes'] = 'Yes'
                        annotator.box_label(xyxy, 'Shoes')
                    elif 'No Shoes' in label:
                        pay_load['Shoes'] = 'No Shoes'
                        annotator.box_label(xyxy, 'No Shoes')
                    elif 'Gloves' in label:
                        pay_load['Gloves'] = 'Yes'
                        annotator.box_label(xyxy, 'Gloves')
                    elif 'No Gloves' in label:
                        pay_load['Gloves'] = 'No Gloves'
                        annotator.box_label(xyxy, 'No Gloves')
                    # annotator.box_label(xyxy, label)
                    self.f_event.fire_event(Event.INFO, 'OFFICE-01', 'ppe-detection', 'PPE_Detection', pay_load)
                im0 = annotator.result()
            if self.stop_evt.is_set():
                    break
            cv2.imshow('detection', im0)
            if cv2.waitKey(5) & 0xFF == 27:
                break
        video.release()
        cv2.destroyAllWindows()

def camera_stream():
    snf = PpeDetection(scenario_name = 'ppe-detection')
    snf.start(camera_name=0)

if __name__ == '__main__':
    # people_taking_picture_detection()
    camera_stream()


import os
import datetime
import json
from enum import Enum
import time

import sys
from pathlib import Path
FILE = Path(__file__).resolve()
SCENARIOS = FILE.parents[0]
ROOT = FILE.parents[1]  # Root directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH
ROOT = Path(os.path.relpath(ROOT, Path.cwd()))  # relative

from rich import print, print_json
from util.general import check_requirements
from util.docker_utils import redis_container_start, redis_python_install

class Event(str, Enum):
    DEBUG = 'DEBUG'
    INFO = 'INFO'
    WARNING = 'WARNING'
    ERROR = 'ERROR'
    CRITICAL = 'CRITICAL'

class EventsEngine():
    def __init__(self, use_redis:bool=False, use_event_hub:bool=False, use_log:bool=True):
        self.use_redis = use_redis
        self.use_event_hub = use_event_hub
        self.use_log = use_log

        # Initialize redis if needed
        if self.use_redis:
            redis_container_start()
            redis_python_install()

            import redis
            self.redis_client = redis.Redis(host='localhost', port=6379, db=0)

            # listen to self events for validation
            self.redis_pubsub = self.redis_client.pubsub(ignore_subscribe_messages=True)
            self.redis_pubsub.subscribe('visionai')

    def fire_event(
        self,
        event_type: Event = Event.INFO,
        camera:str = 'CAMERA??',
        scenario: str = 'SCENARIO??',
        event_name = 'EVENT??',
        event_data = None
        ):

        if self.use_log:
            # Log the event to stdout.
            timestr = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            if event_type == Event.DEBUG:
                clr = 'cyan'
            elif event_type == Event.INFO:
                clr = 'blue'
            elif event_type == Event.WARNING:
                clr = 'yellow'
            elif event_type == Event.ERROR:
                clr = 'dark_red'
            elif event_type == Event.CRITICAL:
                clr = 'bold red'
            else:
                clr = 'grey'

            print(f'[[{clr}]{timestr}[/{clr}]] [[{clr}]{camera}[/{clr}]] [[{clr}]{scenario}[/{clr}]]: [{clr}]{event_name}[/{clr}]')
            print_json(data=event_data)

        if self.use_redis:
            # Log the event to redis
            event_data.update({
                'event_type': event_type,
                'camera': camera,
                'scenario': scenario,
                'event_name': event_name,
                'timestamp': datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            })

            # Send event to "events" channel
            self.redis_client.publish('visionai', json.dumps(event_data))

        if self.use_event_hub:
            # Send the event to Azure Event Hub
            pass

    def print_received_messages(self):
        if self.use_redis:
            print('Got messages:')
            print('- - - - - - - - - - - - - - - - - - - - - - - - - - - ')
            start_time = time.time()
            stop_time = start_time + 2
            while time.time() < stop_time:
                message = self.redis_pubsub.get_message(timeout=1)
                if message is None:
                    time.sleep(0.1)
                    continue

                event_data = json.loads(message['data'])
                print(event_data)
                time.sleep(0.1)
            print('- - - - - - - - - - - - - - - - - - - - - - - - - - - ')

if __name__ == '__main__':

    ev = {
    "version": "0.1",
    "cameras": [
        {
        "id": "581700a9-fb09-4f23-b8b9-abf371930e84",
        "name": "OFFICE-01",
        "uri": "https://youtu.be/whlymAuRtzU",
        "events": "all",
        "preprocess": [
            "face-blurring"
        ],
        "scenarios": [
            "ppe-detection",
            "smoke-and-fire-detection"
        ]
        },
        {
        "id": "581700a9-fb09-4f23-b8b9-abf371930e84",
        "name": "OFFICE-02",
        "uri": "rtsp://wowzaec2demo.streamlock.net/vod/mp4:BigBuckBunny_115k.mp4",
        "events": "critical",
        "preprocess": [
            "face-blurring"
        ],
        "scenarios": [
            "smoke-and-fire-detection"
        ]
        }
    ]
    }


    ee = EventsEngine(use_redis=True)

    while True:
        print('- - - - - - - - - - - - - - - - - - - - - - - - - - - ')
        ee.fire_event(Event.DEBUG, 'OFFICE-01', 'ppe-detection', 'PERSON_WITHOUT_HELMET', ev)
        ee.fire_event(Event.INFO, 'OFFICE-01', 'ppe-detection', 'PERSON_WITHOUT_HELMET', ev)
        ee.fire_event(Event.WARNING, 'OFFICE-01', 'ppe-detection', 'PERSON_WITHOUT_HELMET', ev)
        ee.fire_event(Event.ERROR, 'OFFICE-01', 'ppe-detection', 'PERSON_WITHOUT_HELMET', ev)
        ee.fire_event(Event.CRITICAL, 'OFFICE-01', 'ppe-detection', 'PERSON_WITHOUT_HELMET', ev)

        # ee.print_received_messages()
        print('- - - - - - - - - - - - - - - - - - - - - - - - - - - ')

        time.sleep(1)




import docker
from rich import print
from rich.console import Console
from rich.columns import Columns
from rich.panel import Panel
import time
from urllib.parse import urlparse

import sys
from pathlib import Path
FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

from config import TRITON_HTTP_URL, TRITON_SERVER_DOCKER_IMAGE, TRITON_SERVER_EXEC, TRITON_SERVER_COMMAND, TRITON_MODELS_REPO, TRITON_SERVER_CONTAINER_NAME

class TritonClient():

    # Models start command
    MODELS_SERVE_CMD = '[magenta]visionai models serve[/magenta]'
    MODELS_AVAIL_CMD = '[magenta]visionai models check[/magenta]'
    MODELS_DOWNLOAD_CMD = '[magenta]visionai models download[/magenta]'
    MODELS_STOP_CMD = '[magenta]visionai models stop[/magenta]'
    MODELS_RESTART_CMD = '[magenta]visionai models resatart[/magenta]'
    MODELS_STATUS_CMD = '[magenta]visionai models status[/magenta]'

    def __init__(self) -> None:

        # create client.
        self.docker_client = docker.from_env()

        # console tty
        self.console = Console()

        # Initialize trition client
        self.triton_client = None
        self.init_triton_client()


    def init_triton_client(self):
        # Triton client
        try:
            import tritonclient.http
            parsed_url = urlparse(TRITON_HTTP_URL)
            self.triton_client = tritonclient.http.InferenceServerClient(url=parsed_url.netloc)
            self.triton_client.get_server_metadata()
        except Exception as ex:
            self.triton_client = None

    # Print primitives
    def _get_pretty_container(self, ctainer, image_name=None):
        if ctainer is None:
            return f'None\n[yellow]{image_name}[/yellow]\n[b][red]Not running[/red][/b]'

        id = f'{ctainer.attrs["Id"][:10]}'  # can refer to ctainer by first 10 digits
        name = f'{ctainer.attrs["Name"]}'
        image = f'{ctainer.attrs["Config"]["Image"]}'
        command = ctainer.attrs["Path"] + " " +  " ".join(ctainer.attrs["Args"])
        entrypoint = ctainer.attrs['Config']['Entrypoint']
        command2 = " ".join(ctainer.attrs['Config']['Cmd'])
        status_str = ctainer.attrs['State']['Status']  # "Running" or "Dead" or "Paused"
        status = ctainer.attrs['State']['Running'] # True/False
        started_at = ctainer.attrs['State']['StartedAt'] # 2023-01-29T17:01:13.428140501Z (isoformat string)
        finished_at = ctainer.attrs['State']['FinishedAt'] # 2023-01-29T17:01:13.428140501Z (isoformat string)
        ports = ctainer.attrs['NetworkSettings']['Ports'] # {}
        if status:
            status_str = f'[green][b]{status_str}[/b][/green] since {started_at[:19]}'
        else:
            status_str = f'[red][b]{status_str}[/b][/red] since {finished_at[:19]}'
        ctainer_pretty = f'{id}: [magenta]{name}[/magenta]\n[yellow]{image}[/yellow]\n[blue]{command}[/blue]\n{status_str}'
        return ctainer_pretty

    def _get_pretty_containers(self):
        containers = []
        for ctainer in self.docker_client.containers.list():
            containers.append(Panel(self._get_pretty_container(ctainer), width=50))
        return containers

    def print_pretty_container(self, container, image_name=None):
        '''
        Print a container (with optional image name)
        '''
        if image_name is not None or container is None:
            title = image_name
        else:
            title = container.attrs["Config"]["Image"]

        title = f'Containers for image: [i][yellow]*{title}*[/yellow][/i]'
        self.console.print(Panel(self._get_pretty_container(container, image_name), title=title, expand=False))

    def print_pretty_containers(self):
        '''
        Prettify & print all running containers
        '''
        self.console.print(Panel(Columns(self._get_pretty_containers()), title="Running Containers", expand=False))


    def get_container_by_image_name(self, image_name):
        '''
        Get containers by an image name (incl. partial)
        '''
        for ctainer in self.docker_client.containers.list():
            if image_name.lower() in ctainer.attrs['Config']['Image']:
                return ctainer
        return None

    def is_triton_running(self):
        '''
        Check if triton server is running
        '''
        ctainer = self.get_container_by_image_name(TRITON_SERVER_EXEC)
        if ctainer is not None and self.triton_client is None:
            self.init_triton_client()

        return ctainer is not None

    def _get_pretty_models(self, models):
        ret = []
        for model in models:
            model_name = model['name']

            # Model config
            # Ex:
            # {
            #     'name': 'ppe-detection',
            #     'platform': 'onnxruntime_onnx',
            #     'backend': 'onnxruntime',
            #     'version_policy': {'latest': {'num_versions': 1}},
            #     'max_batch_size': 0,
            #     'input': [
            #         {
            #             'name': 'images',
            #             'data_type': 'TYPE_FP32',
            #             'format': 'FORMAT_NONE',
            #             'dims': [1, 3, 640, 640],
            #             'is_shape_tensor': False,
            #             'allow_ragged_batch': False,
            #             'optional': False
            #         }
            #     ],
            #     'output': [
            #         {
            #             'name': 'output0',
            #             'data_type': 'TYPE_FP32',
            #             'dims': [1, 25200, 14],
            #             'label_filename': 'labels.txt',
            #             'is_shape_tensor': False
            #         }
            #     ],
            #     'batch_input': [],
            #     'batch_output': [],
            #     'optimization': {
            #         'priority': 'PRIORITY_DEFAULT',
            #         'input_pinned_memory': {'enable': True},
            #         'output_pinned_memory': {'enable': True},
            #         'gather_kernel_buffer_threshold': 0,
            #         'eager_batching': False
            #     },
            #     'instance_group': [
            #         {
            #             'name': 'ppe-detection',
            #             'kind': 'KIND_GPU',
            #             'count': 1,
            #             'gpus': [0],
            #             'secondary_devices': [],
            #             'profile': [],
            #             'passive': False,
            #             'host_policy': ''
            #         }
            #     ],
            #     'default_model_filename': 'model.onnx',
            #     'cc_model_filenames': {},
            #     'metric_tags': {},
            #     'parameters': {},
            #     'model_warmup': []
            # }
            model_config = self.triton_client.get_model_config(model_name)
            platform = model_config['backend']  # onnxruntime
            model_input  = 'IN[0] :' + model_config['input'][0]['data_type'] + ' ' + str(model_config['input'][0]['dims'])
            model_output = 'OUT   :' + model_config['output'][0]['data_type'] + ' ' + str(model_config['output'][0]['dims'])

            # Inference stats
            # Ex:
            # {
            #     'model_stats': [
            #         {
            #             'name': 'ppe-detection',
            #             'version': '1',
            #             'last_inference': 0,
            #             'inference_count': 0,
            #             'execution_count': 0,
            #             'inference_stats': {
            #                 'success': {'count': 0, 'ns': 0},
            #                 'fail': {'count': 0, 'ns': 0},
            #                 'queue': {'count': 0, 'ns': 0},
            #                 'compute_input': {'count': 0, 'ns': 0},
            #                 'compute_infer': {'count': 0, 'ns': 0},
            #                 'compute_output': {'count': 0, 'ns': 0},
            #                 'cache_hit': {'count': 0, 'ns': 0},
            #                 'cache_miss': {'count': 0, 'ns': 0}
            #             },
            #             'batch_stats': []
            #         }
            #     ]
            # }
            inf_success = 'INF ✅:' + str(self.triton_client.get_inference_statistics(model_name)['model_stats'][0]['inference_stats']['success'])
            inf_fail    = 'INF ❌:' + str(self.triton_client.get_inference_statistics(model_name)['model_stats'][0]['inference_stats']['fail'])

            model_status = f"[green]{model['state']}[/green]" if model['state'].upper() == 'READY' else f"[red]{model['state']}[/red]"
            model_panel = f"[magenta]{model_name}[/magenta]\n" + \
                f"[blue]version {model['version']}[/blue] - {model_status}\n" + \
                f"[grey50]{model_input}[/grey50]\n" + \
                f"[grey50]{model_output}[/grey50]\n" + \
                f"[green]{inf_success}[/green]\n" + \
                f"[dark_red]{inf_fail}[/dark_red]\n"

            ret.append(Panel(model_panel, title=platform, width=40, expand=False))
        return ret

    def get_models(self):
        '''
        Get models served by Triton container.
        '''

        # get_repository_index() example Output:
        # [
        #     {'name': 'phone-detection', 'version': '1', 'state': 'READY'},
        #     {'name': 'ppe-detection', 'version': '1', 'state': 'READY'},
        #     {'name': 'rust-and-corrosion-detection', 'version': '1', 'state': 'READY'},
        #     {'name': 'smoke-and-fire-detection', 'version': '1', 'state': 'READY'},
        #     {'name': 'smoking-detection', 'version': '1', 'state': 'READY'}
        # ]
        if self.triton_client is None:
            return None
        else:
            return self.triton_client.get_model_repository_index()

    def print_models_served(self):
        '''
        Print models being served by triton server
        '''
        triton_up = self.is_triton_running()
        models = self.get_models()
        if triton_up is False:
            panel_content = Panel(f'[red]ERROR: Model server is not running[/red]\nStart model server with:\n{self.MODELS_SERVE_CMD}', title='Models running:')
        elif models is None or len(models) == 0:
            panel_content = Panel(f'[red]ERROR: No models found![/red]\nCheck model status with:\n{self.MODELS_AVAIL_CMD} \n{self.MODELS_DOWNLOAD_CMD}\n\nYou can also restart the model server with \n{self.MODELS_RESTART_CMD}', title='Models running:')
        else:
            panel_content = Panel(Columns(self._get_pretty_models(models), title='Models running:'))

        self.console.print(panel_content)


    def start_model_server(self):
        '''
        Start the model server

        Implements `visionai models serve` command
        '''

        if self.get_container_by_image_name(TRITON_SERVER_EXEC) is not None:
            print('Model server already running!')
            return

        try:
            print('Pulling docker image (this may take a while)')
            from util.docker_utils import docker_image_pull_with_progress, docker_container_run_triton

            # Stream progress message while pulling the docker image.
            docker_image_pull_with_progress(self.docker_client, image_name=TRITON_SERVER_DOCKER_IMAGE)
            print('Done.')

            # Try starting docker container with NVIDIA runtime,
            # If that is not available - then start the container with regular runtime
            print('Starting model server')
            docker_container_run_triton(
                client=self.docker_client,
                image=TRITON_SERVER_DOCKER_IMAGE,   # image name
                container_name=TRITON_SERVER_CONTAINER_NAME, # container name
                command=TRITON_SERVER_COMMAND,      # command to run in container
                stdout=False,                       # disable logs
                stderr=False,                       # disable stderr logs
                detach=True,                        # detached mode - daemon
                remove=True,                        # remove fs after exit
                auto_remove=True,                   # remove fs if container fails
                device_requests=[                   # similar to --gpus=all ??
                    docker.types.DeviceRequest(capabilities=[['gpu']])
                    ],
                # network_mode='host',                # --net=host
                volumes=                            # -v
                    [f'{TRITON_MODELS_REPO}:/models'],
                ports={
                    '8000': 8000,
                    '8001': 8001,
                    '8002': 8002
                }
            )

            init_complete = False
            init_idx = 0
            for init_idx in range(1,11):
                # sleep a bit
                time.sleep(1)

                # Attach triton_client
                self.init_triton_client()
                if self.triton_client is None:

                    # Check if container still present:
                    triton_container = self.get_container_by_image_name(TRITON_SERVER_EXEC)
                    if triton_container is None:
                        print('Container start failed.')
                        break

                    else:
                        print(f'[{init_idx}/{10}] Triton client init.')
                        continue

                else:
                    init_complete = True
                    break

            if init_complete is True:
                # Everything looks good
                print('Triton client initialized.')
                return True
            else:
                print('[red]- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -[/red]')
                print('[red]Error with starting triton container. (Likely one of the models is corrupt)[/red]')
                print()
                print('[magenta]Ensure the following command is successful:[/magenta]')
                models_repo_path = ROOT / 'models-repo'
                print(f'[magenta]docker run -it -p 8000:8000 -p 8001:8001 -p 8002:8002 -v {models_repo_path}:/models nvcr.io/nvidia/tritonserver:22.12-py3 tritonserver --model-repo /models[/magenta]')
                print('[red]- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -[/red]')
                return False

        except Exception as ex:
            print('ERROR: Trying to start model server.')
            print(f'Exception: {ex}')
            return False

    def stop_model_server(self):
        triton_container = self.get_container_by_image_name(TRITON_SERVER_EXEC)
        if triton_container is None:
            print('ERROR: Model server is not running')

        else:
            try:
                print('Stopping Model Server..')
                triton_container.stop()
                self.triton_client = None
                print('Done.')
            except Exception as ex:
                print(f'ERROR: Unable to stop model-server: {ex}')


if __name__ == '__main__':
    tc = TritonClient()
    # Print all running containers
    tc.print_pretty_containers()

    # Print a single container
    ctainer = tc.get_container_by_image_name(TRITON_SERVER_EXEC)
    tc.print_pretty_container(ctainer, TRITON_SERVER_EXEC)

    # Is triton running?
    triton_check = tc.is_triton_running()
    # print(f'Triton server status: {triton_check}')

    # Models served
    models = tc.get_models()
    # print(f'Models: {models}')

    # Print models
    tc.print_models_served()


    # Start model server
    tc.start_model_server()

    # Print models served
    tc.print_models_served()

    # Stop the model server
    tc.stop_model_server()
    tc.print_models_served()

    # Start the model server again
    tc.start_model_server()
    tc.print_models_served()




import os
import sys
import typer
import time
import json
from uuid import uuid4
from rich import print
from rich.progress import track
from rich.prompt import Prompt
from pathlib import Path
import datetime

FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

from config import CONFIG_FILE

camera_app = typer.Typer()

# Camera app
@camera_app.command('list')
def camera_list():
    '''
    List available cameras

    Print cameras available in the system and the scenarios / routines
    that are set up for them.
    '''
    with open(CONFIG_FILE, 'r') as f:
        config_data = json.load(f)
    print(config_data['cameras'])

@camera_app.command('add')
def camera_add(
    name: str = typer.Option(..., help='Camera Name', prompt=True),
    uri: str = typer.Option(..., help='URI for camera', prompt=True),
    description: str = typer.Option(..., help='Description', prompt=True)
    ):
    '''
    Add a named camera instance

    Add a camera as a named instance in the system. For adding
    a camera we support RTSP, HLS, HTTP(S) systems. To add a camera
    you need to provide a name for the camera, URI for the camera
    (including any username/password within the URI itself),
    description for camera (about its location, where its pointing,
    who is the vendor etc.).

    Before the camera is added - we need to test out if the
    camera instance is valid. We need to be able to read from
    the camera and calculate its FPS. Show this information
    on the screen.
    '''

    print(f'Adding camera : {name}, {uri}, {description}')
    with open(CONFIG_FILE, 'r') as f:
        config_data = json.load(f)

    # Check if name already present
    for camera in config_data['cameras']:
        if camera['name'] == name:
            print(f'Error: Camera {name} already present')
            return

    # TODO: Validate the camera stream is good (check FPS)
    config_data['cameras'].append({
        'id': str(uuid4()),
        'name': name,
        'uri': uri,
        'events': 'all',
        'description': description,
        'preprocess': [],
        'scenarios': []
    })

    with open(CONFIG_FILE, 'w') as f:
        json.dump(config_data, f, indent=2)

    print(f'Successfully added camera: {name}')


@camera_app.command('remove')
def camera_remove(name: str=typer.Option('Camera name', prompt=True, confirmation_prompt=True)):
    '''
    Remove a camera from the system

    Specify a named camera that needs to be removed from
    the system. Once removed, all the scenarios and pre-process
    routines associated with the camera will be removed.
    '''

    print(f'Removing camera : {name}')
    with open(CONFIG_FILE, 'r') as f:
        config_data = json.load(f)

    # Check if name already present
    found = False
    for camera in config_data['cameras']:
        if camera['name'] == name:
            found = True
            config_data['cameras'].remove(camera)
            break

    if found:
        # Write back to JSON if successfully removed.
        with open(CONFIG_FILE, 'w') as f:
            json.dump(config_data, f, indent=4)

        print(f'Successfully removed camera: {name}')
    else:
        print(f'Unable to find camera {name} to remove')

@camera_app.command('reset')
def camera_reset(
    confirm: bool=typer.Option(False, help="Confirm delete", prompt="Are you sure you want to reset camera configuration?")
):
    '''
    Reset all camera configuration.

    All cameras and their scenarios would be removed
    from the system. Any earlier configuration is backed up
    as a timed json backup file.
    '''

    if confirm is True:
        print('Deleting all camera configuration')
        with open(CONFIG_FILE, 'r') as f:
            config_data = json.load(f)

        backup_time = datetime.datetime.now().strftime('%Y-%m-%d-%H-%M-%S')
        with Path(CONFIG_FILE).with_suffix(f'.{backup_time}.backup.json') as backup_file:
            if len(config_data['cameras']) > 0:
                print(f'Backing up current camera configuration at: {backup_file}')
                with open(backup_file, 'w') as f:
                    json.dump(config_data, f, indent=4)

        with open(CONFIG_FILE, 'w') as f:
            config_data = {
                'version': '0.1',
                'cameras': []
                }
            json.dump(config_data, f, indent=4)
        print('Camera configuration reset.')

    else:
        print('Skip resetting camera configuration')

@camera_app.command('preview')
def camera_preview(
    name:str = typer.Option(..., help='camera name to preview')
    ):
    '''
    Preview the camera system

    View the camera feed, review FPS etc available for camera.
    '''
    print(f"TODO: Implement camera preview functionality.")

@camera_app.command('add-scenario')
def camera_add_scenario(
    camera: str = typer.Option(..., help='camera name', prompt=True),
    scenario: str = typer.Option(..., help='scenario name', prompt=True)
    ):
    '''
    Add a scenario for a camera

    Add an individual scenario to be run for a camera. Specify the
    names for scenario and camera.
    '''

    print(f'Adding scenario : {scenario} for camera: {camera}')
    with open(CONFIG_FILE, 'r') as f:
        config_data = json.load(f)

    # find camera
    found_camera = False
    for cam in config_data['cameras']:
        if cam['name'] == camera:
            found_camera = True

            scen_already_present = False
            for scen in cam['scenarios']:
                if scen['name'] == scenario:
                    scen_already_present = True
                    break

            if scen_already_present is False:
                cam['scenarios'].append({
                    'name': scenario,
                    'events': 'all',
                    'schedule': 'all',
                    'focus': 'all'
                })

                with open(CONFIG_FILE, 'w') as f:
                    json.dump(config_data, f, indent=4)

                print(f'Scenario {scenario} successfully added for camera {camera}')

            else:
                print(f'Scenario {scenario} already present for camera {camera}')

            break

    if not found_camera:
        print(f'Error: camera {camera} not available')
        return


@camera_app.command('list-scenarios')
def camera_list_scenarios(
    camera: str = typer.Option(..., help='camera name', prompt=True)
    ):
    '''
    List scenarios for a camera

    List all the scenarios for a camera. Specify the
    names for scenario and camera.
    '''

    print(f'Listing scenarios for camera: {camera}')
    with open(CONFIG_FILE, 'r') as f:
        config_data = json.load(f)

    # find camera
    found_camera = False
    for cam in config_data['cameras']:
        if cam['name'] == camera:
            found_camera = True

            if len(cam['scenarios']) > 0:
                print(f'Available scenarios for camera {camera}:')
                for scen in cam['scenarios']:
                    print(f'  {scen["name"]}')
            else:
                print(f'No scenarios available for camera {camera}')

            break

    if not found_camera:
        print(f'Error: camera {camera} not available')
        return


@camera_app.command('remove-scenario')
def scenario_remove(
    camera: str=typer.Option(..., help='camera', prompt=True),
    scenario: str=typer.Option(..., help='scenario name', prompt=True)
    ):
    '''
    Remove a scenario from a camera

    Specify a named scenario that needs to be removed from
    the system. Once removed, all the scenarios and pre-process
    routines associated with the scenario will be removed.
    '''

    print(f'Removing scenario : {scenario} from camera {camera}')
    with open(CONFIG_FILE, 'r') as f:
        config_data = json.load(f)

    # Check if name already present
    found = False
    for cam in config_data['cameras']:
        for scen in cam['scenarios']:
            if scen['name'] == scenario:
                found = True
                cam['scenarios'].remove(scen)
                break

    if found:
        # Write back to JSON if successfully removed.
        with open(CONFIG_FILE, 'w') as f:
            json.dump(config_data, f, indent=4)

        print(f'Successfully removed scenario: {scenario} from {camera}')
    else:
        print(f'Unable to remove scenario {scenario} from {camera}')


@camera_app.command('list-scenario')
@camera_app.command('list-scenarios')
def scenario_list(
    camera:str = typer.Option('', help="Camera name")
):
    '''
    List scenarios configured for a camera

    '''
    with open(CONFIG_FILE, 'r') as f:
        config_data = json.load(f)

    # if no camera provided, print scenarios for all cameras.
    if camera == '':
        scenarios_list = []
        for cam in config_data['cameras']:
            scenarios_list.append({
                'cam': cam['name'],
                'scenarios': cam['scenarios']
            })

        print(f'scenarios for all cameras:')
        print(scenarios_list)

    else:
        camera_found = False
        scenarios_list = []
        for cam in config_data['cameras']:
            if cam['name'] == camera:
                camera_found = True
                scenarios_list.append({
                    'cam': camera,
                    'scenarios': cam['scenarios']
                })
                break
        if camera_found:
            print(f'scenarios for camera: {camera}')
            print(scenarios_list)
        else:
            print(f'camera not found: {camera}')


@camera_app.callback()
def callback():
    '''
    Manage cameras

    An organization can have multiple cameras that are installed
    at different places. They may be from different vendors and/or
    maybe using different security surveillance software. Most
    cameras however do support RTSP, RTMP or HLS streams as an
    output. Please refer to your camera vendor documentation to
    find this out. This module will help you onboard those cameras
    on visionai systems by using a simple named instance for each
    camera.
    '''

if __name__ == '__main__':
    camera_app()




import typer

device_app = typer.Typer()

# Device app
@device_app.command('list')
def device_list():
    '''
    List available devices

    Get a list of all available [processing] devices
    '''
    devices = ['oos-training', 'edge-dev1', 'edge-dev2']
    print(f'Devices : {devices}')

@device_app.command('select')
def device_select(device: str):
    '''
    Select a device

    Not sure why is this needed at this time.
    '''
    print(f'Selecting device : {device}')

@device_app.command('modules')
def device_modules():
    '''
    List running modules on the device

    Again this does not make much sense at this time. Let's revisit.
    '''
    modules = ['edgeCtl', 'edgeEvents']
    print(f'Listing modules : {modules}')

@device_app.command('stats')
def device_gpu_mem_stats():
    '''
    Machine health (GPU/Mem stats)

    Show machine health (GPU/memory stats). This can be used to
    determine if more scenarios can be run on the machine or not.
    '''
    print('Getting gpu/mem stats')


@device_app.callback()
def callback():
    '''
    Manage device features

    Since scenarios run on individual edge-devices, and we don't
    have enough control over the CPU, Memory, GPU statistics - it is
    imperative that we have strong methods for validating if a scenario
    can run on a chosen platform. This module provides many utilities
    to check CPU, Memory and GPU statistics for the edge device. We also
    provide an Azure Managed service where these scenarios can be
    configured and run on your premise on pre-validated VM machines.
   '''
if __name__ == '__main__':
    device_app()



import os
import sys
from pathlib import Path
import unittest

FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
PKGDIR = FILE.parents[2] # visionai dir
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

from config import VISIONAI_EXEC
from util.general import WorkingDirectory, invoke_cmd

class TestInvokeCliWeb(unittest.TestCase):
    @WorkingDirectory(PKGDIR)
    def setUp(self):
        # Initialize visionai package before running tests
        output = invoke_cmd(f'{VISIONAI_EXEC} init')

    @WorkingDirectory(PKGDIR)
    def test_invoke_web_help_command(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} web --help')
        assert 'install' not in output
        assert 'start' in output
        assert 'status' in output
        assert 'stop' in output


    @WorkingDirectory(PKGDIR)
    def test_invoke_web_start_stop_status_command(self):
        # Stop anything that may be running
        output = invoke_cmd(f'{VISIONAI_EXEC} web stop')
        assert 'Stop web-app' in output

        # Start web-services
        output = invoke_cmd(f'{VISIONAI_EXEC} web start')
        assert 'Starting web app at' in output
        assert 'Starting web service API' in output
        assert 'Starting redis server' in output
        assert 'Starting grafana server' in output
        # assert 'Grafana server is at' in output  # CI/CD failing, TODO: Fix this
        assert 'Redis server is at' in output
        assert 'Web service API available at' in output
        assert 'Web app available at' in output

        # Start web-services again. It should just print the status
        output = invoke_cmd(f'{VISIONAI_EXEC} web start')
        assert 'Starting web app at' not in output
        assert 'Starting web service API' not in output
        assert 'Starting redis server' not in output
        # assert 'Starting grafana server' not in output # TODO: Fix this, failing in CI/CD
        # assert 'Grafana server is at' in output #TODO: Fix this, failing in CI/CD
        assert 'Redis server is at' in output
        assert 'Web service API available at' in output
        assert 'Web app available at' in output

        # Stop web-services
        output = invoke_cmd(f'{VISIONAI_EXEC} web stop')
        assert 'Stop web-app' in output
        assert 'Stop API service' in output
        assert 'Stop redis server' in output
        # assert 'Stop grafana server' in output # TODO: Fix this, failing in CI/CD

        # Stop web-services again. It should just print it is not running
        output = invoke_cmd(f'{VISIONAI_EXEC} web stop')
        assert 'Web-app not running' in output
        assert 'Web-API not running' in output
        assert 'Redis not running' in output
        # assert 'Grafana not running' in output # TODO: Fix this, failing in CI/CD


if __name__ == '__main__':
    unittest.main()



import os
import contextlib
import platform
import threading
from pathlib import Path

def emojis(str=''):
    # Return platform-dependent emoji-safe version of string
    return str.encode().decode('ascii', 'ignore') if platform.system() == 'Windows' else str


class TryExcept(contextlib.ContextDecorator):
    # YOLOv5 TryExcept class. Usage: @TryExcept() decorator or 'with TryExcept():' context manager
    def __init__(self, msg=''):
        self.msg = msg

    def __enter__(self):
        pass

    def __exit__(self, exc_type, value, traceback):
        if value:
            print(emojis(f"{self.msg}{': ' if self.msg else ''}{value}"))
        return True

def threaded(func):
    # Multi-threads a target function and returns thread. Usage: @threaded decorator
    def wrapper(*args, **kwargs):
        thread = threading.Thread(target=func, args=args, kwargs=kwargs, daemon=True)
        thread.start()
        return thread

    return wrapper


def join_threads(verbose=False):
    # Join all daemon threads, i.e. atexit.register(lambda: join_threads())
    main_thread = threading.current_thread()
    for t in threading.enumerate():
        if t is not main_thread:
            if verbose:
                print(f'Joining thread {t.name}')
            t.join()

class WorkingDirectory(contextlib.ContextDecorator):
    # Usage: @WorkingDirectory(dir) decorator or 'with WorkingDirectory(dir):' context manager
    def __init__(self, new_dir):
        self.dir = new_dir  # new dir
        Path(self.dir).mkdir(parents=True, exist_ok=True) # create if not exists
        self.cwd = Path.cwd().resolve()  # current dir

    def __enter__(self):
        os.chdir(self.dir)

    def __exit__(self, exc_type, exc_val, exc_tb):
        os.chdir(self.cwd)



from rich import print
import time
import cv2
from PIL import Image
import numpy as np
import math
from math import atan
import sys
from pathlib import Path
FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH
from models.plots import Annotator
from util.general import LOGGER
from scenarios import Scenario
from config import TRITON_HTTP_URL
from events.events_engine import EventsEngine
from enum import Enum

class Event(str, Enum):
    DEBUG = 'DEBUG'
    INFO = 'INFO'
    WARNING = 'WARNING'
    ERROR = 'ERROR'
    CRITICAL = 'CRITICAL'

class ErgonomicsDetection(Scenario):
    model = None
    conf_thres=0.25  # confidence threshold
    iou_thres=0.45  # NMS IOU threshold
    classes=None  # filter by class: --class 0, or --class 0 2 3
    agnostic_nms=False  # class-agnostic NMS
    max_det=1000  # maximum detections per image
    line_thickness=3  # bounding box thickness (pixels)
    color = (0, 200, 0)
    thickness = 1 
    device='cpu'  # cuda device, i.e. 0 or 0,1,2,3 or cpu

    def __init__(self, scenario_name, camera_name=0, events=None, triton_url=TRITON_HTTP_URL):
        self.up_count = 0
        self.down_count = 0
        self.up_counter = True
        self.down_counter = False
        from util.general import check_requirements
        check_requirements('mediapipe', install=True)
        import mediapipe as mp
        self.mp_drawing = mp.solutions.drawing_utils
        self.mp_drawing_styles = mp.solutions.drawing_styles
        self.mp_pose = mp.solutions.pose
        self.f_event =  EventsEngine(use_redis=True)
        from models.triton_client_yolov5 import yolov5_triton
        self.model = yolov5_triton('http://localhost:8000', scenario_name)
        super().__init__(scenario_name, camera_name, events, triton_url)

    def land_3d(self, landmarks_3d):
        left_shoulder = np.array([landmarks_3d.landmark[11].x, landmarks_3d.landmark[11].y, landmarks_3d.landmark[11].z])
        right_shoulder = np.array([landmarks_3d.landmark[12].x, landmarks_3d.landmark[12].y, landmarks_3d.landmark[12].z])
        left_hip = np.array([landmarks_3d.landmark[23].x, landmarks_3d.landmark[23].y, landmarks_3d.landmark[23].z])
        right_hip = np.array([landmarks_3d.landmark[24].x, landmarks_3d.landmark[24].y, landmarks_3d.landmark[24].z])
        left_knee = np.array([landmarks_3d.landmark[25].x, landmarks_3d.landmark[25].y, landmarks_3d.landmark[25].z])
        right_knee = np.array([landmarks_3d.landmark[26].x, landmarks_3d.landmark[26].y, landmarks_3d.landmark[26].z])
        left_ankle = np.array([landmarks_3d.landmark[27].x, landmarks_3d.landmark[27].y, landmarks_3d.landmark[27].z])
        right_ankle = np.array([landmarks_3d.landmark[28].x, landmarks_3d.landmark[28].y, landmarks_3d.landmark[28].z])
        mid_shoulder = (left_shoulder + right_shoulder) / 2
        mid_hip = (left_hip + right_hip) / 2
        mid_knee = (left_knee + right_knee) / 2
        mid_ankle = (left_ankle + right_ankle) / 2
        s_w_k_angle = self.get_angle(mid_knee, mid_hip, mid_shoulder, mid_hip, adjust=True)
        w_k_a_angle = self.get_angle(mid_ankle, mid_knee, mid_hip, mid_knee, adjust=True)
        return s_w_k_angle, w_k_a_angle

    def get_angle(self, a, b, c, d, adjust):
        """return the angle between two vectors"""
        vec1 = a - b
        vec2 = c - d

        cosine_angle = np.dot(vec1, vec2) / (np.linalg.norm(vec1) * np.linalg.norm(vec2))
        angle = np.arccos(cosine_angle)
        if (adjust):
            angle_adjusted = abs(np.degrees(angle) - 180)
            return int(angle_adjusted)
        else:
            return int(abs(np.degrees(angle)))

    def point_check(self, dim, p) : # bl, tr bottom_left, top_right, point
        bl = [dim[0],dim[3]]
        tr = [dim[2],dim[1]]
        # print(bl, tr, p)
        if (p[0] > bl[0] and p[0] < tr[0] and p[1] < bl[1] and p[1] > tr[1]) :
            return True
        else :
            return False

    def findAngle(self, M1, M2):
        try:
            PI = 3.14159265
            angle = abs((M2 - M1) / (1 + M1 * M2))
            ret = atan(angle)
            val = (ret * 180) / PI
            return round(val, 4)
        except:
            return 0.0

    def draw_box(self, image, land_marks):
        cv2.line(image, land_marks[11], land_marks[12], self.color, self.thickness)
        cv2.line(image, land_marks[11], land_marks[23], self.color, self.thickness)
        cv2.line(image, land_marks[12], land_marks[24], self.color, self.thickness)
        cv2.line(image, land_marks[23], land_marks[24], self.color, self.thickness)
        cv2.line(image, land_marks[23], land_marks[25], self.color, self.thickness)
        cv2.line(image, land_marks[24], land_marks[26], self.color, self.thickness)
        cv2.line(image, land_marks[25], land_marks[27], self.color, self.thickness)
        cv2.line(image, land_marks[26], land_marks[28], self.color, self.thickness)
        return image

    def slope(self, x, y):
        if(y[0] - x[0] != 0):
            return (float)(y[1]-x[1])/(y[0]-x[0])
        return 0.0

    def find_angle(self, land_marks):
        ind = [[11,27,2,23],[5,12,24,28]]
        angles = []
        slopee = []
        sl = self.slope(land_marks[11],land_marks[27])
        ag = self.findAngle(sl, 0)
        return ag


    def start(self, camera_name=0):
        '''
        Stream processing
        When running a scenario - the caller can specify any specific camera.
        '''
        import cv2
        stream = camera_name
        print(f'Opening capture for {stream}')
        video = cv2.VideoCapture(stream)
        while True:
            # Do processing
            ret, image = video.read()
            if ret is False:
                LOGGER.error('ERROR: reading from video frame')
                time.sleep(1)
                continue
            start_time = time.time()
            text_font = 1
            text_thickness = 1
            centroid = []
            top_center = []
            # Load model
            device = 'cpu'
            results = self.model(image, size=640) #, visualize=False)
            stride, names= self.model.stride, self.model.names
            im0 = image
            seen = 0
            det = results.xyxy[0]
            # seen += 1
            annotator = Annotator(im0, line_width=self.line_thickness) #, example=str(names))
            person_count = 0
            if len(det):
                count_i = 0
                param = {"person_id": [], "pose_points":[], "dim":[], "mobile_centeroid": []}
                for *xyxy, conf, cls in reversed(det):
                    c = int(cls)  # integer class
                    label = f'{names[c]} {conf:.2f}'
                    # print("Detected labels : ", label)
                    if 'class0' in label:
                        person_status = "Normal"
                        crop = im0[int(xyxy[1]):int(xyxy[3]),int(xyxy[0]):int(xyxy[2])]
                        # param["person_id"].append(person_count)
                        with self.mp_pose.Pose(
                            min_detection_confidence=0.5,
                            min_tracking_confidence=0.5) as pose:
                            crop = cv2.cvtColor(crop, cv2.COLOR_BGR2RGB)
                            results = pose.process(crop)
                            crop= cv2.cvtColor(crop, cv2.COLOR_RGB2BGR)
                            land_marks = []
                            if results.pose_landmarks:
                                for idx, pose_marks in enumerate(results.pose_landmarks.landmark):
                                    h,w = int(xyxy[3]-xyxy[1]), int(xyxy[2]-xyxy[0])
                                    cx,cy = int(pose_marks.x * w) , int(pose_marks.y * h)
                                    land_marks.append([cx,cy])
                                if land_marks != []:
                                    crop = self.draw_box(crop, land_marks)
                                    landmark_3d = results.pose_world_landmarks
                                    mid_s_w_k_angle, mid_w_k_a_angle = self.land_3d(landmark_3d)
                                    # slopeangle = self.find_angle(land_marks)
                                    if mid_s_w_k_angle > 75  and self.down_counter == True:
                                        self.down_count = self.down_count+1
                                        self.up_counter = True
                                        self.down_counter = False
                                    elif mid_s_w_k_angle < 75 and self.up_counter == True:
                                        if self.up_counter == True:
                                            self.up_count = self.up_count+1
                                            self.up_counter = False
                                            self.down_counter = True
                                    cv2.putText(crop, "BdCnt:" + str(self.down_count), (0,50) , cv2.FONT_HERSHEY_SIMPLEX, text_font, (0,0,250), text_thickness, cv2.LINE_AA)
                                    self.f_event.fire_event(Event.INFO, 'OFFICE-01', 'ergonomics-detection', 'BENDING_COUNT', {"Bending Count" : self.down_count})
                                    im0[int(xyxy[1]):int(xyxy[3]),int(xyxy[0]):int(xyxy[2])] = crop
                            param["pose_points"].append(land_marks)
                            param["dim"].append([int(xyxy[0]),int(xyxy[1]),int(xyxy[2]),int(xyxy[3])])
                            annotator.box_label(xyxy, label)
                        person_count+=1
                im0 = annotator.result()
            if self.stop_evt.is_set():
                    break
            cv2.imshow('detection', im0)
            if cv2.waitKey(5) & 0xFF == 27:
                break
        video.release()
        cv2.destroyAllWindows()

def ergonomics_detection():
    snf = ErgonomicsDetection(scenario_name = 'phone-detection')
    # snf.load_model()
    img = cv2.imread("visionai/models/data/images/fall_12.png")
    # img = Image.open('visionai/models/data/images/no_fall.jpg')
    # img = Path('visionai/models/data/images/no_fall.jpg')
    img  = snf.process_image(img)
    cv2.imwrite('sfd_res_2.jpg', img)

def camera_stream():
    snf = ErgonomicsDetection(scenario_name = 'ergonomics-detection')
    snf.start(camera_name=0)

if __name__ == '__main__':
    # people_taking_picture_detection()
    camera_stream()




import os
import sys
from pathlib import Path
import unittest

FILE = Path(__file__).resolve()
ROOT = FILE.parents[1] # visionai/visionai
PKGDIR = FILE.parents[2] # visionai folder
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

from config import VISIONAI_EXEC
from util.general import WorkingDirectory, invoke_cmd

class TestInvokeCliModels(unittest.TestCase):
    @WorkingDirectory(PKGDIR)
    def setUp(self):
        # Initialize visionai package before running tests
        output = invoke_cmd(f'{VISIONAI_EXEC} init')

    @WorkingDirectory(PKGDIR)
    def test_invoke_models_status(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} models status')
        assert 'Models status' in output

    @WorkingDirectory(PKGDIR)
    def test_invoke_models_start(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} models start')
        assert 'Start models' in output

    @WorkingDirectory(PKGDIR)
    def test_invoke_models_start(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} models serve')
        assert 'Start models' in output

    @WorkingDirectory(PKGDIR)
    def test_invoke_models_stop(self):
        output = invoke_cmd(f'{VISIONAI_EXEC} models stop')
        assert 'Stop models' in output

if __name__ == '__main__':
    unittest.main()



import os
import sys
from pathlib import Path
import unittest
import docker
import pytest

FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
PKGDIR = FILE.parents[2] # visionai dir
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

from config import *
from util.docker_utils import *
from util.general import WorkingDirectory, invoke_cmd



class TestRedis(unittest.TestCase):

    @pytest.mark.skip(reason="Grafana keeps failing on CI/CD. Skipping for now.")
    def test_redis_grafana_functions(self):
        # Stop any priori containers
        redis_container_stop()
        grafana_container_stop()

        # Start containers
        redis_container_start()
        grafana_container_start()

        # Install redis python package
        redis_python_install()

        # Validate redis python package installed
        assert redis_python_package_installed(), 'Redis python package is not installed.'

        # Check if containers is running
        assert docker_container_is_running(container_name=REDIS_CONTAINER_NAME), 'Redis container is not running.'
        assert docker_container_is_running(container_name=GRAFANA_CONTAINER_NAME), 'Grafana container is not running.'

        # Check if network is created
        assert docker_network_is_running(network_name=DOCKER_NETWORK), 'Docker network is not created.'


        # Check containers are joined to that network
        client = docker.from_env()
        container = client.containers.get(REDIS_CONTAINER_NAME)
        networks = container.attrs['NetworkSettings']['Networks']
        redis_network_joined = False
        if networks:
            for network_name, network_config in networks.items():
                if network_name == DOCKER_NETWORK:
                    redis_network_joined = True

        container = client.containers.get(GRAFANA_CONTAINER_NAME)
        networks = container.attrs['NetworkSettings']['Networks']
        grafana_network_joined = False
        if networks:
            for network_name, network_config in networks.items():
                if network_name == DOCKER_NETWORK:
                    grafana_network_joined = True

        assert redis_network_joined, f'Redis container is not joined to network {DOCKER_NETWORK}'
        assert grafana_network_joined, f'Grafana container is not joined to network {DOCKER_NETWORK}'

        # Stop the containers
        redis_container_stop()
        grafana_container_stop()

        # Ensure containers are stopped
        assert docker_container_is_running(container_name=REDIS_CONTAINER_NAME) == False, 'Redis container is still running.'
        assert docker_container_is_running(container_name=GRAFANA_CONTAINER_NAME) == False, 'Grafana container is still running.'


if __name__ == '__main__':
    unittest.main()


